from .init_main import Framework
from .version import VERSION

# 2024.06.13 
# 잘못된 설계로 인해 import 만으로 초기화 되버려 lib을 사용할 수 없다.
# 분리.

F = None
frame = None
logger = None
app = None
celery = None
db = None
scheduler = None
socketio = None
rd = None
path_app_root = None
path_data = None
get_logger = None
SystemModelSetting = None

def initiaize():
    global F
    global frame 
    global logger
    global app
    global celery
    global db
    global scheduler
    global socketio
    global path_app_root
    global path_data
    global get_logger
    global SystemModelSetting

    F = Framework.get_instance()
    frame = F
    logger = frame.logger
    app = frame.app
    celery = frame.celery
    db = frame.db
    scheduler = frame.scheduler
    socketio = frame.socketio
    rd = frame.rd
    path_app_root = frame.path_app_root
    path_data = frame.path_data
    get_logger = frame.get_logger

    frame.initialize_system()
    from system.setup import SystemModelSetting as SS
    SystemModelSetting = SS
    frame.initialize_plugin()
    return frame

from flask_login import login_required
from support import d

from .init_declare import User, check_api
from .scheduler import Job



from flask_login import current_user, login_user, logout_user

from .setup import *

name = 'route'

class ModuleRoute(PluginModuleBase):
    def __init__(self, P):
        super(ModuleRoute, self).__init__(P, name=name)
        self.route()

    # 일반적인 routing에서는 login_required 때문에 /system/login으로 redirect 됨.
    def route(self):
        @P.blueprint.route('/login', methods=['GET', 'POST'])
        def login():
            if F.SystemModelSetting.get_bool('use_login') == False:
                username = F.SystemModelSetting.get('web_id')
                F.users[username].authenticated = True
                login_user(frame.users[username], remember=True)
                #current_user = USERS[username]
                return redirect(request.args.get("next"))
            return render_template(f'system_login.html', next=request.args.get("next"))

        @P.blueprint.route('/ajax/login/<cmd>', methods=['POST'])
        def command(cmd):
            if cmd == 'command':
                return self.process_command(request.form['command'], request.form.get('arg1'), request.form.get('arg2'), request.form.get('arg3'), request)

        @P.blueprint.route('/logout', methods=['GET'])
        def logout():
            current_user.authenticated = False
            logout_user()
            return redirect('/')

        @F.login_manager.user_loader
        def user_loader(user_id):
            return F.users[user_id]


        @P.blueprint.route('/restart', methods=['GET'])
        @login_required
        def restart():
            threading.Timer(1, F.restart).start()
            return render_template('system_restart.html',sub='restart', referer=request.headers.get("Referer"))
        

        @P.blueprint.route('/shutdown', methods=['GET'])
        @login_required
        def shutdown():
            threading.Timer(1, F.shutdown).start()
            return render_template('system_restart.html',sub='shutdown', referer=request.headers.get("Referer"))


        @F.socketio.on('connect', namespace=f'/{P.package_name}/restart')
        def restart_socket_connect():
            F.socketio.emit('connect', {}, namespace='/{P.package_name}/restart')


    def process_menu(self, page, req):
        return render_template('sample.html', title=f"{__package__}/{name}/{page}")
        #return render_template(f'{__package__}_{name}.html', arg={})

    def process_command(self, command, arg1, arg2, arg3, req):
        if command == 'login':
            username = arg1
            password = arg2
            remember = (arg3 == 'true')
            if username not in frame.users:
                return jsonify('no_id')
            elif not F.users[username].can_login(password):
                return jsonify('wrong_password')
            else:
                F.users[username].authenticated = True
                login_user(frame.users[username], remember=remember)
                return jsonify('redirect')
            



import os
import traceback

from framework import F


class Util(object):

    @staticmethod
    def db_to_dict(db_list):
        ret = []
        for item in db_list:
            ret.append(item.as_dict())
        return ret

    @staticmethod
    def get_paging_info(count, current_page, page_size):
        try:
            paging = {}
            paging['prev_page'] = True
            paging['next_page'] = True
            if current_page <= 10:
                paging['prev_page'] = False
            
            paging['total_page'] = int(count / page_size) + 1
            if count % page_size == 0:
                paging['total_page'] -= 1
            paging['start_page'] = int((current_page-1)/10) * 10 + 1
            paging['last_page'] = paging['total_page'] if paging['start_page'] + 9 > paging['total_page'] else paging['start_page'] + 9
            if paging['last_page'] == paging['total_page']:
                paging['next_page'] = False
            paging['current_page'] = current_page
            paging['count'] = count
            F.logger.debug('paging : c:%s %s %s %s %s %s', count, paging['total_page'], paging['prev_page'], paging['next_page'] , paging['start_page'], paging['last_page'])
            return paging
        except Exception as e:
            F.logger.debug(f"Exception:{str(e)}")
            F.logger.debug(traceback.format_exc())
    

    # 토렌트 인포에서 최대 크기 파일과 폴더명을 리턴한다
    @staticmethod
    def get_max_size_fileinfo(torrent_info):
        try:
            ret = {}
            max_size = -1
            max_filename = None
            for t in torrent_info['files']:
                if t['size'] > max_size:
                    max_size = t['size']
                    max_filename = str(t['path'])
            t = max_filename.split('/')
            ret['filename'] = t[-1]
            if len(t) == 1:
                ret['dirname'] = ''
            elif len(t) == 2:
                ret['dirname'] = t[0]
            else:
                ret['dirname'] = max_filename.replace('/%s' % ret['filename'], '')
            ret['max_size'] = max_size
            return ret
        except Exception as e: 
            F.logger.error(f"Exception:{str(e)}")
            F.logger.error(traceback.format_exc())


    # 압축할 폴더 경로를 인자로 받음. 폴더명.zip 생성
    @staticmethod
    def makezip(zip_path, zip_extension='zip'):
        import zipfile
        try:
            if os.path.isdir(zip_path):
                zipfilename = os.path.join(os.path.dirname(zip_path), '%s.%s' % (os.path.basename(zip_path), zip_extension))
                fantasy_zip = zipfile.ZipFile(zipfilename, 'w')
                for f in os.listdir(zip_path):
                    #if f.endswith('.jpg') or f.endswith('.png'):
                        src = os.path.join(zip_path, f)
                        fantasy_zip.write(src, os.path.basename(src), compress_type = zipfile.ZIP_DEFLATED)
                fantasy_zip.close()
            import shutil
            shutil.rmtree(zip_path)
            return True
        except Exception as e:
            F.logger.error(f"Exception:{str(e)}")
            F.logger.error(traceback.format_exc())
        return False



    @staticmethod
    def make_apikey(url):
        from framework import SystemModelSetting
        url = url.format(ddns=SystemModelSetting.get('ddns'))
        if SystemModelSetting.get_bool('use_apikey'):
            if url.find('?') == -1:
                url += '?'
            else:
                url += '&'
            url += 'apikey=%s' % SystemModelSetting.get('apikey')
        return url






import shutil

from support import SupportFile

from .setup import *

name = 'all_log'

class ModuleLog(PluginModuleBase):
    def __init__(self, P):
        super(ModuleLog, self).__init__(P, name=name, first_menu='list')
        

    def process_menu(self, page, req):
        try:
            arg = {}
            log_files = os.listdir(os.path.join(F.config['path_data'], 'log'))
            log_files.sort()
            log_list = []
            for x in log_files:
                if x.endswith('.log'):
                    log_list.append(x)
            arg['log_list'] = '|'.join(log_list)
            arg['all_list'] = '|'.join(log_files)
            arg['filename'] = 'all.log'
           
            if 'filename' in request.form:
                arg['filename'] = request.form['filename']
            arg['filename'] = req.args.get('filename', arg['filename'])
            return render_template(f'{__package__}_{name}.html', arg=arg)
        except Exception as e: 
            P.logger.error(f'Exception:{str(e)}')
            P.logger.error(traceback.format_exc())
            return render_template('sample.html', title=f"{__package__}/{name}/{page}")


    def process_command(self, command, arg1, arg2, arg3, req):
        ret = {'ret':'success'}
        if command == 'plugin_install':
            ret = F.PluginManager.plugin_install(arg1)
        elif command == 'get_plugin_list':
            data = []
            """
            for name, entity in F.PluginManager.plugin_list.items():
                if entity['version'] == '3':
                    data.append({'package_name':name})
                else:
                    data.append(entity['P'].plugin_info)
            """
            for name, entity in F.PluginManager.all_package_list.items():
                try:
                    if entity.get('version') == '3':
                        #data.append(entity)
                        data.append({'package_name':name})
                    else:
                        data.append(entity['P'].plugin_info)
                        data[-1]['loading'] = entity.get('loading')
                        data[-1]['status'] = entity.get('status')
                        data[-1]['log'] = entity.get('log')
                except Exception as e:
                    data.append({'package_name':name})
                    P.logger.error(f'Exception:{str(e)}')
                    P.logger.error(traceback.format_exc())
            ret['data'] = data
            #P.logger.debug(data)
        elif command == 'uninstall':
            info = F.PluginManager.all_package_list[arg1]
            if os.path.exists(info['path']):
                try:
                    shutil.rmtree(info['path'])
                    ret['msg'] = '삭제하였습니다.<br>재시작시 적용됩니다.'
                except Exception as e:
                    P.logger.error(f'Exception:{str(e)}')
                    P.logger.error(traceback.format_exc())
                    ret['msg'] = info['path'] + "<br>삭제에 실패하였습니다.<br>" + str(e)
                    ret['ret'] = 'danger'
            else:
                ret['msg'] = info['path'] + "<br>폴더가 없습니다."
                ret['ret'] = 'danger'
        return jsonify(ret)


    def plugin_load(self):
        try:
            pass
        except Exception as e:
            P.logger.error(f'Exception:{str(e)}')
            P.logger.error(traceback.format_exc())


    


import os
import threading
import time
import traceback

from framework import F
from support import SupportSubprocess


class ToolModalCommand(object):
    __thread = None
    __title = None
    __commands = None
    __clear = None
    __show_modal = None
    __wait = None
    __ss_process = None
    __abort = None
    __return_log = None

    @classmethod
    def start(cls, title, commands, clear=True, wait=False, show_modal=True):
        if cls.__thread != None:
            cls.__abort = True
            cls.__thread = None
        if cls.__ss_process != None:
            cls.__ss_process.process_close()
            cls.__ss_process = None
        cls.__title = title
        cls.__commands = commands
        cls.__clear = clear
        cls.__wait = wait
        cls.__show_modal = show_modal
        cls.__thread = None
        cls.__abort = False
        cls.__return_log = ''
        return cls.__start()



    @classmethod
    def __start(cls):
        try:
            if cls.__show_modal:
                if cls.__clear:
                    F.socketio.emit("command_modal_clear", None, namespace='/framework')
            cls.__thread = threading.Thread(target=cls.__execute_thread_function, args=())
            cls.__thread.setDaemon(True)
            cls.__thread.start()
            if cls.__wait:
                time.sleep(1)
                cls.__thread.join()
                return cls.__return_log
        except Exception as e: 
            F.logger.error(f"Exception:{str(e)}")
            F.logger.error(traceback.format_exc())

    @classmethod
    def __execute_thread_function(cls):
        try:
            if cls.__show_modal:
                F.socketio.emit("command_modal_show", cls.__title, namespace='/framework')
                F.socketio.emit("loading_hide", None, namespace='/framework')
                
            for command in cls.__commands:
                if cls.__abort:
                    return
                if command[0] == 'msg':
                    if cls.__show_modal:
                        F.socketio.emit("command_modal_add_text", '%s\n\n' % command[1], namespace='/framework')
                elif command[0] == 'system':
                    if cls.__show_modal:
                        F.socketio.emit("command_modal_add_text", '$ %s\n\n' % command[1], namespace='/framework')
                    os.system(command[1])
                else:
                    #show_command = True
                    #if command[0] == 'hide':
                    #    show_command = False
                    #    command = command[1:]
                    cls.__ss_process = SupportSubprocess(command, stdout_callback=cls.process_callback, callback_line=False)
                    cls.__ss_process.start()
                    cls.__ss_process.process_close()
                    cls.__ss_process = None
                time.sleep(1)
        except Exception as exception: 
            if cls.__show_modal:
                F.socketio.emit("command_modal_show", cls.__title, namespace='/framework')
                F.socketio.emit("command_modal_add_text", str(exception), namespace='/framework')
                F.socketio.emit("command_modal_add_text", str(traceback.format_exc()), namespace='/framework')

    @classmethod
    def process_callback(cls, call_id, mode, text):
        #F.logger.warning(text)
        if cls.__show_modal == False:
            return
        if mode == 'end':
            F.socketio.emit("command_modal_add_text", "\n\n<<프로세스 종료>>", namespace='/framework')
            F.socketio.emit("command_modal_input_disable", "", namespace='/framework')
        elif mode == 'thread_end':
            #F.socketio.emit("command_modal_add_text", "\n\n<<프로세스 종료>>", namespace='/framework')
            F.socketio.emit("command_modal_input_disable", "", namespace='/framework')
        else:
            if text != None:
                cls.__return_log += text
                F.socketio.emit("command_modal_add_text", text, namespace='/framework')

    
    @classmethod
    def modal_close(cls):
        if cls.__thread != None:
            cls.__abort = True
            
        if cls.__ss_process != None:
            cls.__ss_process.process_close()
            cls.__ss_process = None

    @classmethod
    def input_command(cls, cmd):
        if cls.__ss_process != None:
            cls.__ss_process.input_command(cmd)


    @classmethod
    def send_message(cls, text):
        F.socketio.emit("command_modal_add_text", '%s\n\n' % text, namespace='/framework')



import traceback
from datetime import datetime

from framework import F
from support import SupportDiscord, SupportTelegram, SupportYaml, SupportSlack

from . import logger


class ToolNotify(object):

    @classmethod 
    def send_message(cls, text, message_id=None, image_url=None):
        if F.SystemModelSetting.get_bool('notify_advaned_use'):
            return cls.send_advanced_message(text, image_url=image_url, message_id=message_id)
        else:
            if F.SystemModelSetting.get_bool('notify_telegram_use'):
                SupportTelegram.send_telegram_message(text, image_url=image_url, bot_token=F.SystemModelSetting.get('notify_telegram_token'), chat_id=F.SystemModelSetting.get('notify_telegram_chat_id'))
            if F.SystemModelSetting.get_bool('notify_discord_use'):
                SupportDiscord.send_discord_message(text, image_url=image_url, webhook_url=F.SystemModelSetting.get('notify_discord_webhook')) 
            if F.SystemModelSetting.get_bool('notify_slack_use'):
                SupportSlack.send_slack_message(text, image_url=image_url, webhook_url=F.SystemModelSetting.get('notify_slack_webhook'))


    @classmethod
    def send_advanced_message(cls, text, image_url=None, policy=None, message_id=None): 
        try:
            message_id = message_id.strip()
            policy = SupportYaml.read_yaml(F.config['notify_yaml_filepath'])
            if message_id is None or message_id not in policy:
                message_id = 'DEFAULT'
            now = datetime.now()
            for item in policy[message_id]:
                if item.get('enable_time') != None:
                    tmp = item.get('enable_time').split('-')
                    if now.hour < int(tmp[0]) or now.hour > int(tmp[1]):
                        continue
                if item.get('type') == 'telegram':
                    if item.get('token', '') == '' or item.get('chat_id', '') == '':
                        continue
                    SupportTelegram.send_telegram_message(text, image_url=image_url, bot_token=item.get('token'), chat_id=item.get('chat_id'), disable_notification=item.get('disable_notification', False))
                elif item.get('type') == 'discord':
                    if item.get('webhook', '') == '':
                        continue
                    SupportDiscord.send_discord_message(text, image_url=image_url, webhook_url=item.get('webhook'))
                elif item.get('type') == 'slack':
                    if item.get('webhook', '') == '':
                        continue
                    SupportSlack.send_slack_message(text, image_url=image_url, webhook_url=item.get('webhook'))
            return True
        except Exception as e: 
            logger.error(f"Exception:{str(e)}")
            logger.error(traceback.format_exc()) 
        return False



import argparse
import os
import sys

sys.path.append(os.path.join(os.path.dirname(os.path.dirname(__file__)), 'lib'))

from support import SupportFile, logger


class ChangeUI:
    def change(self, target, old, new):
        if target == 'macro':
            target_filepath = os.path.join(os.path.dirname(os.path.dirname(__file__)), 'lib', 'framework', 'templates', 'macro.html')
            text = SupportFile.read_file(target_filepath)
            SupportFile.write_file(target_filepath, text.replace(old, new))

    def process_args(self):
        parser = argparse.ArgumentParser()
        parser.add_argument('--target', default='macro')
        parser.add_argument('--old', default='btn-outline-primary')
        parser.add_argument('--new', required=True)
        args = parser.parse_args()
        self.change(args.target, args.old, args.new)

if __name__== "__main__":
    ChangeUI().process_args()



import codecs
import json
import os
import re
import traceback
import zipfile

from . import logger


class SupportFile(object):

    @classmethod
    def read_file(cls, filename, mode='r'):
        try:
            ifp = codecs.open(filename, mode, encoding='utf8')
            data = ifp.read()
            ifp.close()
            return data
        except Exception as e: 
            logger.error(f"Exception:{str(e)}")
            logger.error(traceback.format_exc())

    @classmethod
    def write_file(cls, filename, data, mode='w'):
        try:
            import codecs
            ofp = codecs.open(filename, mode, encoding='utf8')
            ofp.write(data)
            ofp.close()
        except Exception as e: 
            logger.error(f"Exception:{str(e)}")
            logger.error(traceback.format_exc()) 

    @classmethod
    def read_json(cls, filepath):
        try:
            with open(filepath, "r", encoding='utf8') as json_file:
                data = json.load(json_file)
                return data
        except Exception as e: 
            logger.error(f"Exception:{str(e)}")
            logger.error(traceback.format_exc()) 

    @classmethod
    def write_json(cls, filepath, data):
        try:
            if os.path.dirname(filepath) != '':
                os.makedirs(os.path.dirname(filepath), exist_ok=True)
            with open(filepath, "w", encoding='utf8') as json_file:
                json.dump(data, json_file, indent=4, ensure_ascii=False)
        except Exception as e: 
            logger.error(f"Exception:{str(e)}")
            logger.error(traceback.format_exc()) 

    



    @classmethod
    def download_file(cls, url, filepath):
        try:
            headers = {
                'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/71.0.3578.98 Safari/537.36',
                'Accept' : 'text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,image/apng,*/*;q=0.8',
                'Accept-Language' : 'ko-KR,ko;q=0.9,en-US;q=0.8,en;q=0.7',
                'Connection': 'Keep-Alive',
            }

            import requests
            with open(filepath, "wb") as file_is:   # open in binary mode
                response = requests.get(url, headers=headers)               # get request
                file_is.write(response.content)      # write to file
                return True
        except Exception as e:
            logger.debug(f"Exception:{str(e)}")
            logger.debug(traceback.format_exc())   
        return False


    @classmethod
    def text_for_filename(cls, text):
        #text = text.replace('/', '')
        # 2021-07-31 X:X
        #text = text.replace(':', ' ')
        text = re.sub('[\\/:*?\"<>|：]', ' ', text).strip()
        text = re.sub("\s{2,}", ' ', text)
        return text


    @classmethod
    def unzip(cls, zip_filepath, extract_folderpath):
        with zipfile.ZipFile(zip_filepath, 'r') as zip_ref:
            zip_ref.extractall(extract_folderpath)


    # 파일처리에서 사용. 중복이면 시간값
    @classmethod
    def file_move(cls, source_path, target_dir, target_filename):
        try:
            import shutil
            import time
            if os.path.exists(target_dir) == False:
                os.makedirs(target_dir)
            target_path = os.path.join(target_dir, target_filename)
            if source_path != target_path:
                if os.path.exists(target_path):
                    tmp = os.path.splitext(target_filename)
                    new_target_filename = f"{tmp[0]} {str(time.time()).split('.')[0]}{tmp[1]}"
                    target_path = os.path.join(target_dir, new_target_filename)
                shutil.move(source_path, target_path)
        except Exception as e:
            logger.debug(f"Exception:{str(e)}")
            logger.debug(traceback.format_exc())

    
    @classmethod
    def size(cls, start_path = '.'):
        if os.path.exists(start_path):
            if os.path.isdir(start_path):
                total_size = 0
                for dirpath, dirnames, filenames in os.walk(start_path):
                    for f in filenames:
                        fp = os.path.join(dirpath, f)
                        if not os.path.islink(fp):
                            total_size += os.path.getsize(fp)
                return total_size
            else:
                return os.path.getsize(start_path)
        return 0


    @classmethod
    def size_info(cls, start_path = '.'):
        ret = {
            'size':0,
            'file_count':0,
            'folder_count':0.
        }
        for dirpath, dirnames, filenames in os.walk(start_path):
            for f in filenames:
                fp = os.path.join(dirpath, f)
                if not os.path.islink(fp):
                    ret['size'] += os.path.getsize(fp)
            ret['folder_count'] += len(dirnames)
            ret['file_count'] += len(filenames)
        return ret


    @classmethod
    def rmtree(cls, folderpath):
        import shutil
        try:
            for root, dirs, files in os.walk(folderpath):
                for name in files:
                    os.remove(os.path.join(root, name))
                for name in dirs:
                    shutil.rmtree(os.path.join(root, name))
            shutil.rmtree(folderpath)
        except Exception as e:
            logger.debug(f"Exception:{str(e)}")
            logger.debug(traceback.format_exc())
            return False


    @classmethod
    def file_move(cls, source_path, target_dir, target_filename):
        try:
            import shutil
            import time
            os.makedirs(target_dir, exist_ok=True)
            target_path = os.path.join(target_dir, target_filename)
            if source_path != target_path:
                if os.path.exists(target_path):
                    tmp = os.path.splitext(target_filename)
                    new_target_filename = f"{tmp[0]} {str(time.time()).split('.')[0]}{tmp[1]}"
                    target_path = os.path.join(target_dir, new_target_filename)
                shutil.move(source_path, target_path)
        except Exception as e:
            logger.debug(f"Exception:{str(e)}")
            logger.debug(traceback.format_exc())





















































































    """
    @classmethod
    def makezip(cls, zip_path, zip_folder=None, zip_extension='zip', remove_folder=False):
        import zipfile
        try:
            zip_path = zip_path.rstrip('/')
            if zip_folder is None:
                zip_folder = os.path.dirname(zip_path)
            elif zip_folder == 'tmp':
                from framework import path_data
                zip_folder = os.path.join(path_data, 'tmp')
            if os.path.isdir(zip_path):
                zipfilepath = os.path.join(zip_folder, f"{os.path.basename(zip_path)}.{zip_extension}")
                fantasy_zip = zipfile.ZipFile(zipfilepath, 'w')
                for f in os.listdir(zip_path):
                    src = os.path.join(zip_path, f)
                    fantasy_zip.write(src, os.path.basename(src), compress_type = zipfile.ZIP_DEFLATED)
                fantasy_zip.close()
            if remove_folder:
                import shutil
                shutil.rmtree(zip_path)
            return zipfilepath
        except Exception as e:
            logger.error(f"Exception:{str(e)}")
            logger.error(traceback.format_exc())
        return
    """

    """
    @classmethod
    def rmtree(cls, folderpath):
        import shutil
        try:
            shutil.rmtree(folderpath)
            return True
        except:
            try:
                os.system("rm -rf '{folderpath}'")
                return True
            except:
                return False
    """     
                

          
    

    

    
    

    @classmethod
    def write_binary(cls, filename, data):
        try:
            with open(filename, 'wb') as f:
                f.write(data)
        except Exception as e: 
            logger.error(f"Exception:{str(e)}")
            logger.error(traceback.format_exc()) 






    @classmethod
    def makezip(cls, zip_path, zip_extension='zip', remove_zip_path=True):
        import shutil
        import zipfile
        try:
            if os.path.exists(zip_path) == False:
                return False
            zipfilepath = os.path.join(os.path.dirname(zip_path), f"{os.path.basename(zip_path)}.{zip_extension}")
            if os.path.exists(zipfilepath):
                return True
            zip = zipfile.ZipFile(zipfilepath, 'w')
            for f in os.listdir(zip_path):
                src = os.path.join(zip_path, f)
                zip.write(src, f, compress_type = zipfile.ZIP_DEFLATED)
            zip.close()
            if remove_zip_path:
                shutil.rmtree(zip_path)
            return zipfilepath
        except Exception as e:
            logger.error(f'Exception:{str(e)}')
            logger.error(traceback.format_exc())
        return None
    

    @classmethod
    def write_yaml(cls, filepath, data):
        import yaml
        with open(filepath, 'w', encoding='utf8') as f:
            yaml.dump(data, f, default_flow_style=False, allow_unicode=True)

    

    @classmethod
    def makezip_all(cls, zip_path, zip_filepath=None, zip_extension='zip', remove_zip_path=True):
        import shutil
        import zipfile
        from pathlib import Path
        try:
            if os.path.exists(zip_path) == False:
                return False
            if zip_filepath == None:
                zipfilepath = os.path.join(os.path.dirname(zip_path), f"{os.path.basename(zip_path)}.{zip_extension}")
            if os.path.exists(zipfilepath):
                os.remove(zipfilepath)
            zip = zipfile.ZipFile(zipfilepath, 'w')
            for file_path in Path(zip_path).rglob("*"):
                zip.write(file_path, file_path.name)


            for (path, dir, files) in os.walk(zip_path):
                for file in files:
                    zip.write(os.path.join(path.replace(zip_path+'/', '').replace(zip_path+'\\', ''), file), compress_type=zipfile.ZIP_DEFLATED)
            zip.close()
            if remove_zip_path:
                shutil.rmtree(zip_path)
            return zipfilepath
        except Exception as e:
            logger.error(f'Exception:{str(e)}')
            logger.error(traceback.format_exc())
        return None

    """
    
    @classmethod
    def read(cls, filepath, mode='r'):
        try:
            import codecs
            ifp = codecs.open(filepath, mode, encoding='utf8')
            data = ifp.read()
            ifp.close()
            if isinstance(data, bytes):
                data = data.decode('utf-8') 
            return data
        except Exception as e: 
            logger.error(f"Exception:{str(e)}")
            logger.error(traceback.format_exc())
    
    """



import shutil

from support import SupportFile, SupportYaml

from .setup import *

name = 'plugin'

class ModulePlugin(PluginModuleBase):
    
    def __init__(self, P):
        super(ModulePlugin, self).__init__(P, name=name, first_menu='list')
        self.all_plugin_list = None
        

    def process_menu(self, page, req):
        arg = P.ModelSetting.to_dict()
        try:
            return render_template(f'{__package__}_{name}_{page}.html', arg=arg)
        except Exception as e: 
            P.logger.error(f'Exception:{str(e)}')
            P.logger.error(traceback.format_exc())
            return render_template('sample.html', title=f"{__package__}/{name}/{page}")


    def process_command(self, command, arg1, arg2, arg3, req):
        ret = {'ret':'success'}
        if command == 'plugin_install':
            ret = F.PluginManager.plugin_install(arg1)
        elif command == 'get_plugin_list':
            data = []
            """
            for name, entity in F.PluginManager.plugin_list.items():
                if entity['version'] == '3':
                    data.append({'package_name':name})
                else:
                    data.append(entity['P'].plugin_info)
            """
            for name, entity in F.PluginManager.all_package_list.items():
                try:
                    if 'P' in entity:
                        data.append(entity['P'].plugin_info)
                        data[-1]['loading'] = entity.get('loading')
                        data[-1]['status'] = entity.get('status')
                        data[-1]['log'] = entity.get('log')
                    else:
                        data.append({'package_name':name})
                        data[-1]['loading'] = entity.get('loading')
                        data[-1]['status'] = entity.get('status')
                        data[-1]['log'] = entity.get('log')
                except Exception as e:
                    data.append({'package_name':name})
                    P.logger.error(f'Exception:{str(e)}')
                    P.logger.error(traceback.format_exc())
            ret['data'] = data
            #P.logger.debug(data)
        elif command == 'uninstall':
            info = F.PluginManager.all_package_list[arg1]
            if os.path.exists(info['path']):
                try:
                    shutil.rmtree(info['path'])
                    ret['msg'] = '삭제하였습니다.<br>재시작시 적용됩니다.'
                except Exception as e:
                    P.logger.error(f'Exception:{str(e)}')
                    P.logger.error(traceback.format_exc())
                    ret['msg'] = info['path'] + "<br>삭제에 실패하였습니다.<br>" + str(e)
                    ret['ret'] = 'danger'
            else:
                ret['msg'] = info['path'] + "<br>폴더가 없습니다."
                ret['ret'] = 'danger'
        elif command == 'get_plugin_list_all':
            if self.all_plugin_list == None:
                filepath = os.path.join(os.path.dirname(__file__), 'files', 'all_plugin.yaml')
                self.all_plugin_list = SupportYaml.read_yaml(filepath)
                
                def get_plugin(_name):
                    for _cate in self.all_plugin_list:
                        for _plugin in _cate['list']:
                            if _plugin['package_name'] == _name:
                                P.logger.info(_name)
                                if _name == 'ff_reystream':
                                    P.logger.info(_name)
                                return _plugin
                            
                
                for name, entity in F.PluginManager.all_package_list.items():
                    try:
                        _plugin = get_plugin(name)
                        if _plugin != None:
                            _plugin['loading'] = entity.get('loading')
                            _plugin['status'] = entity.get('status')
                            _plugin['log'] = entity.get('log')
                            
                            _plugin['version'] = entity['P'].plugin_info['version']
                            
                    except Exception as e:
                        data.append({'package_name':name})
                        P.logger.error(f'Exception:{str(e)}')
                        P.logger.error(traceback.format_exc())
            ret['data'] = self.all_plugin_list
        return jsonify(ret)


    def plugin_load(self):
        try:
            pass
        except Exception as e:
            P.logger.error(f'Exception:{str(e)}')
            P.logger.error(traceback.format_exc())


    


import os
import sys
import traceback
from io import BytesIO

import requests

from . import logger


class SupportImage(object):

    @classmethod
    def horizontal_to_vertical(cls, url):
        try:
            from PIL import Image
            im = Image.open(requests.get(url, stream=True).raw)
            width,height = im.size
            new_height = int(width * 1.5)
            new_im = Image.new('RGB', (width, new_height))
            new_im.paste(im, (0, int((new_height-height)/2)))

            img_byte_arr = BytesIO()
            new_im.save(img_byte_arr, format='PNG')
            img_byte_arr = img_byte_arr.getvalue()
            from . import SupportDiscord
            return SupportDiscord.discord_proxy_image_bytes(img_byte_arr)
        except Exception as e: 
            logger.error(f"Exception:{str(e)}")
            logger.error(traceback.format_exc())




import platform

from support import SupportUtil

from .setup import *

name = 'home'

class ModuleHome(PluginModuleBase):
    info_thread = None 

    def __init__(self, P):
        super(ModuleHome, self).__init__(P, name=name)
        default_route_socketio_module(self)

    def process_menu(self, page, req):
        arg = {'changelog':F.config['DEFINE']['CHANGELOG']}
        return render_template(f'{__package__}_{name}.html', info=self.get_info('static'), arg=arg)

    def process_command(self, command, arg1, arg2, arg3, req):
        if command == 'recent_version':
            result = F.get_recent_version()
            if result:
                #F.config['recent_version'] = '4.0.0'
                ret = {'msg': f"최신버전 : {F.config['recent_version']}", 'type':'success'}
            else:
                ret = {'msg': f"확인 실패", 'type':'warning'}
            return jsonify(ret)
        elif command == 'get_config':
            data = {}
            for key, value in F.app.config.items():
                if key not in ['SECRET_KEY']:
                    data[key] = str(value)
            ret = {'json':{'Framework':F.config, 'Flask':data}, 'title':'config'}
            return jsonify(ret)
       

    def socketio_connect(self):
        self.send_info()
        if self.info_thread != None:
            return

        def func():
            while True:
                if len(self.socketio_list) == 0:
                    break
                self.send_info()
                time.sleep(1)
            self.info_thread = None

        self.info_thread = threading.Thread(target=func, args=())
        self.info_thread.daemon = True
        self.info_thread.start()

    def send_info(self):
        ret = {}
        ret['system'] = self.get_info()
        ret['scheduler'] = scheduler.get_job_list_info()
        F.socketio.emit("status", ret, namespace=f'/{P.package_name}/{name}')
    
    
    def get_info(self, mode=''):
        info = {}
        if mode == 'static':
            info['platform'] = platform.platform()
            info['processor'] = platform.processor()

            info['python_version'] = sys.version
            info['version'] = VERSION
            info['recent_version'] = F.config['recent_version']
            info['path_app'] = F.config['path_app']
            info['path_data'] = F.config['path_data']
            info['path_working'] = F.config['path_working']
            info['config_filepath'] = F.config['config_filepath']
            info['running_type'] = F.config['running_type'] 
            info['use_celery'] = '사용' if F.config['use_celery'] else '미사용'
        else:
            info['version'] = VERSION
            info['recent_version'] = F.config['recent_version']
            #info['auth'] = frame.config['member']['auth_desc']
            info['cpu_percent'] = 'not supported'
            info['memory'] = 'not supported'
            info['disk'] = 'not supported'
            if frame.config['running_type'] != 'termux':
                try:
                    import psutil
                    info['cpu_percent'] = '%s %%' % psutil.cpu_percent() 
                    tmp = psutil.virtual_memory()
                    info['memory'] = [
                        SupportUtil.sizeof_fmt(tmp[0], suffix='B'),
                        SupportUtil.sizeof_fmt(tmp[3], suffix='B'),
                        SupportUtil.sizeof_fmt(tmp[1], suffix='B'),
                        tmp[2]
                    ]
                except:
                    pass

                try:
                    if platform.system() == 'Windows':
                        s = os.path.splitdrive(path_app_root)
                        root = s[0]
                    else:
                        root = '/'
                    tmp = psutil.disk_usage(root)
                    info['disk'] = [
                        SupportUtil.sizeof_fmt(tmp[0], suffix='B'), 
                        SupportUtil.sizeof_fmt(tmp[1], suffix='B'), 
                        SupportUtil.sizeof_fmt(tmp[2], suffix='B'), 
                        tmp[3], root]
                except Exception as exception: 
                    pass
            try:
                system_start_time = SystemModelSetting.get('system_start_time')
                timedelta = datetime.now() - datetime.strptime(system_start_time, '%Y-%m-%d %H:%M:%S')
                info['time'] = [
                    system_start_time,
                    str(timedelta).split('.')[0],
                    F.config['arg_repeat']
                ]
            except Exception as exception: 
                info['time'] = str(exception)
        return info



import os
import platform
import shutil
import sys
import threading
import traceback
import zipfile

import requests
from framework import F
from support import SupportFile, SupportSubprocess, SupportYaml


class PluginManager:
    plugin_list = {}
    plugin_menus = {}
    setting_menus = []
    all_package_list = {}


    @classmethod 
    def get_plugin_name_list(cls):
        plugins = []
        #2019-07-17 
        try:
            plugin_path = os.path.join(F.config['path_data'], 'plugins')
            if os.path.exists(plugin_path) == True and os.path.isdir(plugin_path) == True:
                sys.path.insert(1, plugin_path)
                tmps = os.listdir(plugin_path)
                add_plugin_list = []
                for t in tmps:
                    if t.startswith('_') == False and t.startswith('.') == False and os.path.isdir(os.path.join(plugin_path, t)) and t != 'false' and t != 'tmp':
                        add_plugin_list.append(t)
                        cls.all_package_list[t] = {'pos':'normal', 'path':os.path.join(plugin_path, t), 'loading':(F.config.get('plugin_loading_only_devpath', None) != True)}

                plugins = plugins + add_plugin_list
        except Exception as e:
            F.logger.error(f"Exception:{str(e)}")
            F.logger.error(traceback.format_exc())
        
        if F.config.get('plugin_loading_only_devpath', None) == True:
            plugins = []

        # 2018-09-04
        try:
            #plugin_path = F.SystemModelSetting.get('plugin_dev_path')
            plugin_path = F.config['path_dev']
            plugin_path_list = []
            if type(plugin_path) == type(''):
                plugin_path_list = [plugin_path]
            elif type(plugin_path) == type([]):
                plugin_path_list = plugin_path

            for __ in plugin_path_list:
                if __ != None and __ != '':
                    if os.path.exists(__):
                        sys.path.insert(0, __)
                        tmps = os.listdir(__)
                        add_plugin_list = []
                        for t in tmps:
                            if t.startswith('_') == False and t.startswith('.') == False and os.path.isdir(os.path.join(__, t)) and t != 'false' and t != 'tmp':
                                add_plugin_list.append(t)
                                cls.all_package_list[t] = {'pos':'dev', 'path':os.path.join(__, t), 'loading':True}
                        plugins = plugins + add_plugin_list
        except Exception as e:
            F.logger.error(f"Exception:{str(e)}")
            F.logger.error(traceback.format_exc())

        # plugin_loading_list
        try:
            plugin_loading_list = F.config.get('plugin_loading_list', None)
            if plugin_loading_list != None and (type(plugin_loading_list) == type([]) and len(plugin_loading_list)) > 0:
                new_plugins = []
                for _ in plugins:
                    if _ in plugin_loading_list:
                        new_plugins.append(_)
                    else:
                        cls.all_package_list[_]['loading'] = False
                        cls.all_package_list[_]['status'] = 'not_include_loading_list'
                plugins = new_plugins
        except Exception as e:
            F.logger.error(f"Exception:{str(e)}")
            F.logger.error(traceback.format_exc())

        # plugin_except_list
        try:
            plugin_except_list = F.config.get('plugin_except_list', None)
            if plugin_except_list != None and (type(plugin_except_list) == type([]) and len(plugin_except_list)) > 0:
                new_plugins = []
                for _ in plugins:
                    if _ not in plugin_except_list:
                        new_plugins.append(_)
                    else:
                        cls.all_package_list[_]['loading'] = False
                        cls.all_package_list[_]['status'] = 'include_except_list'
                plugins = new_plugins
        except Exception as e:
            F.logger.error(f"Exception:{str(e)}")
            F.logger.error(traceback.format_exc())
        return plugins



    # menu, blueprint, plugin_info, plugin_load, plugin_unload
    @classmethod
    def plugin_init(cls):
        try:
            plugins = cls.get_plugin_name_list()
            plugins = sorted(plugins)

            F.logger.debug(plugins)
            for plugin_name in plugins:
                F.logger.debug(f'[+] PLUGIN LOADING Start.. [{plugin_name}]') 
                entity = cls.all_package_list[plugin_name]
                try:
                    try:
                        mod = __import__(f'{plugin_name}.setup', fromlist=['setup'])
                    except Exception as e:
                        F.logger.error(f'Exception:{str(e)}')
                        F.logger.error(traceback.format_exc())
                        F.logger.warning(f'[!] NOT normal plugin : [{plugin_name}]')
                        continue

                    try:
                        entity['setup_mod'] = mod
                        entity['P'] = getattr(mod, 'P')
                        mod_blue_print = getattr(entity['P'], 'blueprint')
                        if mod_blue_print: 
                            F.app.register_blueprint(mod_blue_print)
                    except Exception as exception:
                        F.logger.warning(f'[!] BLUEPRINT not exist : [{plugin_name}]') 
                    cls.plugin_list[plugin_name] = entity
                except Exception as e:
                    F.logger.error(f"Exception:{str(e)}")
                    F.logger.error(traceback.format_exc())
                    F.logger.debug('no blueprint')
                    cls.all_package_list[plugin_name]['loading'] = False
                    cls.all_package_list[plugin_name]['status'] = 'import fail'
                    cls.all_package_list[plugin_name]['log'] = traceback.format_exc()
            
            
            if not F.config['run_celery']:
                try:
                    with F.app.app_context(): 
                        F.db.create_all()
                except Exception as e:
                    F.logger.error(f"Exception:{str(e)}")
                    F.logger.error(traceback.format_exc())
                    F.logger.debug('db.create_all error')

            for key, entity in cls.plugin_list.items():
                try:
                    mod_plugin_load = getattr(entity['P'], 'plugin_load_celery')
                    if mod_plugin_load:
                        def func(mod_plugin_load, key):
                            try:
                                #F.logger.debug(f'[!] plugin_load_celery threading start : [{key}]') 
                                mod_plugin_load()
                                #F.logger.debug(f'[!] plugin_load_celery threading end : [{key}]') 
                            except Exception as e:
                                F.logger.error(f"Exception:{str(e)}")
                                F.logger.error(traceback.format_exc())
                        t = threading.Thread(target=func, args=(mod_plugin_load, key))
                        t.setDaemon(True)
                        t.start()
                except Exception as e:
                    F.logger.error(f"Exception:{str(e)}")
                    F.logger.error(traceback.format_exc())

            if not F.config['run_flask']:
                return
            
            for key, entity in cls.plugin_list.items():
                try:
                    mod_plugin_load = getattr(entity['P'], 'plugin_load')
                    if mod_plugin_load:
                        def func(mod_plugin_load, key):
                            try:
                                F.logger.info(f'[!] plugin_load threading start : [{key}]') 
                                mod_plugin_load()
                                F.logger.debug(f'[!] plugin_load threading end : [{key}]') 
                            except Exception as e:
                                F.logger.error('### plugin_load exception : %s', key)
                                F.logger.error(f"Exception:{str(e)}")
                                F.logger.error(traceback.format_exc())
                                cls.all_package_list[key]['loading'] = False
                                cls.all_package_list[key]['status'] = 'plugin_load error'
                                cls.all_package_list[key]['log'] = traceback.format_exc()

                                if key in cls.plugin_menus:
                                    del cls.plugin_menus[key]
                                    F.logger.info(f"플러그인 로딩 실패로 메뉴 삭제1 : {key}")
                                    from framework.init_menu import MenuManager
                                    MenuManager.init_menu()
                                    F.logger.info(f"플러그인 로딩 실패로 메뉴 삭제2 : {key}")

                        t = threading.Thread(target=func, args=(mod_plugin_load, key))
                        t.setDaemon(True)
                        t.start()

                except Exception as e:
                    F.logger.debug(f'[!] PLUGIN_LOAD function not exist : [{key}]') 

                try:
                    mod_menu = getattr(entity['P'], 'menu')
                    if mod_menu and cls.all_package_list[key]['loading'] != False:
                        cls.plugin_menus[key]=  {'menu':mod_menu, 'match':False}
                    setting_menu = getattr(entity['P'], 'setting_menu')
                    setting_menu['plugin'] = entity['P'].package_name
                    if setting_menu != None and cls.all_package_list[key]['loading'] != False:
                        F.logger.info(f"확장 설정 : {key}")
                        cls.setting_menus.append(setting_menu)
                except Exception as exception:
                    F.logger.debug('no menu')
            F.logger.debug('### plugin_load threading all start.. : %s ', len(cls.plugin_list))
            # 모든 모듈을 로드한 이후에 app 등록, table 생성, start

        except Exception as e:
            F.logger.error(f"Exception:{str(e)}")
            F.logger.error(traceback.format_exc())



    @classmethod
    def plugin_unload(cls):
        for key, entity in cls.plugin_list.items():
            try:
                mod_plugin_unload = getattr(entity['P'], 'plugin_unload')
                if mod_plugin_unload:
                    mod_plugin_unload()
            except Exception as e:
                F.logger.error('module:%s', key)
                F.logger.error(f'Exception:{str(e)}')
                F.logger.error(traceback.format_exc())
        try:
            from system.setup import P
            P.plugin_unload()
        except Exception as e:
            F.logger.error(f'Exception:{str(e)}')
            F.logger.error(traceback.format_exc())

    @classmethod
    def plugin_install(cls, plugin_git, zip_url=None, zip_filename=None):
        plugin_git = plugin_git.strip()
        is_git = True if plugin_git != None and plugin_git != '' else False
        ret = {}
        try:
            if is_git:
                name = plugin_git.split('/')[-1]
            else:
                name = zip_filename.split('.')[0]
            
            plugin_all_path = os.path.join(F.config['path_data'], 'plugins')
            os.makedirs(plugin_all_path, exist_ok=True)
            plugin_path = os.path.join(plugin_all_path, name)
            plugin_info = None
            if os.path.exists(plugin_path):
                ret['ret'] = 'danger'
                ret['msg'] = '이미 설치되어 있습니다.'
                ret['status'] = 'already_exist'
                return ret
            
            if plugin_git and plugin_git.startswith('http'):
                for tag in ['main', 'master']:
                    try:
                        info_url = plugin_git.replace('github.com', 'raw.githubusercontent.com') + f'/{tag}/info.yaml'
                        plugin_info = requests.get(info_url).json()
                        if plugin_info is not None:
                            break
                    except:
                        pass

            if zip_filename and zip_filename != '':
                zip_filepath = os.path.join(F.config['path_data'], 'tmp', zip_filename)
                extract_filepath = os.path.join(F.config['path_data'], 'tmp', name)
                if SupportFile.download(zip_url, zip_filepath):
                    with zipfile.ZipFile(zip_filepath, 'r') as zip_ref:
                        zip_ref.extractall(extract_filepath)
                    plugin_info_filepath = os.path.join(extract_filepath, 'info.yaml')
                    if os.path.exists(plugin_info_filepath):
                        plugin_info = SupportYaml.read_yaml(plugin_info_filepath)
            
            if plugin_info == None:
                plugin_info = {}

            flag = True
            tmp = plugin_info.get('require_os', '')
            if tmp != '' and type(tmp) == type([]) and platform.system() not in tmp:
                ret['ret'] = 'danger'
                ret['msg'] = '설치 가능한 OS가 아닙니다.'
                ret['status'] = 'not_support_os'
                flag = False
            
            tmp = plugin_info.get('require_running_type', '')
            if tmp != '' and type(tmp) == type([]) and F.config['running_type'] not in tmp:
                ret['ret'] = 'danger'
                ret['msg'] = '설치 가능한 실행타입이 아닙니다.'
                ret['status'] = 'not_support_running_type'
                flag = False


            if flag:
                if plugin_git and plugin_git.startswith('http'):
                    command = ['git', '-C', plugin_all_path, 'clone', plugin_git + '.git', '--depth', '1']
                    log = SupportSubprocess.execute_command_return(command, log=True)
                    F.logger.debug(log)
                    if os.path.exists(plugin_path):
                        ret['ret'] = 'success'
                        ret['msg'] = '정상적으로 설치하였습니다. 재시작시 적용됩니다.<br>' + '<br>'.join(log['log'].split('\n'))
                    else:
                        ret['ret'] = 'danger'
                        ret['msg'] = '설치 실패.<br>' + '<br>'.join(log['log'].split('\n'))
                if zip_filename and zip_filename != '':
                    
                    if os.path.exists(plugin_path) == False:
                        shutil.move(extract_filepath, plugin_path)
                    else:
                        for tmp in os.listdir(extract_filepath):
                            shutil.move(os.path.join(extract_filepath, tmp), plugin_path)
                    log = ''

                # 2021-12-31
                tmp = plugin_info.get('require_plugin', '')
                if tmp != '' and type(tmp) == type([]) and len(tmp) > 0:
                    for need_plugin in plugin_info['require_plugin']:
                        if need_plugin['package_name'] in cls.plugin_init:
                            F.logger.debug(f"Dependency 설치 - 이미 설치됨 : {need_plugin['package_name']}")
                            continue
                        else:
                            F.logger.debug(f"Dependency 설치 : {need_plugin['package_name']}")
                            cls.plugin_install(need_plugin['home'], None, None)

                
                #ret['msg'] = '<br>'.join(log)
                   
        except Exception as e: 
            F.logger.error(f'Exception:{str(e)}')
            F.logger.error(traceback.format_exc())
            ret['ret'] = 'danger'
            ret['msg'] = str(e)
        return ret


    @classmethod
    def plugin_update(cls):
        try:
            if os.environ.get('UPDATE_STOP') == 'true':
                return
            if os.environ.get('PLUGIN_UPDATE_FROM_PYTHON') == 'false':
                return
            if F.config['plugin_update'] != True:
                return
            plugins_path = os.path.join(F.config['path_data'], 'plugins')
            if os.path.exists(plugins_path) == False:
                return
            tmps = os.listdir(plugins_path)
            for t in tmps:
                plugin_path = os.path.join(plugins_path, t)
                if t.startswith('_') or t.startswith('.'):
                    continue
                if os.path.exists(os.path.join(plugin_path, '.git')):
                    command = ['git', '-C', plugin_path, 'reset', '--hard', 'HEAD']
                    ret = SupportSubprocess.execute_command_return(command)
                    F.logger.debug(ret)
                    command = ['git', '-C', plugin_path, 'pull']
                    ret = SupportSubprocess.execute_command_return(command)
                    F.logger.debug(ret)
                else:
                    F.logger.debug(f"{plugin_path} not git repo")
        except Exception as e: 
            F.logger.error(f"Exception:{str(e)}")
            F.logger.error(traceback.format_exc())


    @classmethod
    def get_plugin_instance(cls, package_name):
        try:
            if cls.all_package_list[package_name]['loading']:
                return cls.all_package_list[package_name]['P']
        except:
            pass



"""
import os, sys, traceback, re, threading, time
from datetime import datetime, timedelta
from flask import Blueprint, render_template, jsonify, redirect, request
from framework import frame, F, login_required, check_api, Job, SystemModelSetting
from plugin import PluginModuleBase, get_model_setting, Logic, default_route, create_plugin_instance
"""



import threading
import time
import traceback

from framework import F, Job

#########################################################


class Logic(object):
    db_default = {
        'recent_menu_plugin' : '',
    }

    def __init__(self, P):
        self.P = P

    def plugin_load(self):
        self.P.logger.debug('%s plugin_load', self.P.package_name)
        self.db_init()
        for module in self.P.module_list:
            module.migration()
            if module.page_list is not None:
                for page_instance in module.page_list:
                    page_instance.migration()

        for module in self.P.module_list:
            module.plugin_load()
            if module.page_list is not None:
                for page_instance in module.page_list:
                    page_instance.plugin_load()
        if self.P.ModelSetting is not None:
            for module in self.P.module_list:
                key = f'{module.name}_auto_start'
                key2 = f'{module.name}_interval'
                if self.P.ModelSetting.has_key(key) and self.P.ModelSetting.get_bool(key) and self.P.ModelSetting.has_key(key2):
                    self.scheduler_start(module.name)
                
                if module.page_list is not None:
                    for page_instance in module.page_list:
                        key = f'{module.name}_{page_instance.name}_auto_start'
                        if self.P.ModelSetting.has_key(key) and self.P.ModelSetting.get_bool(key):
                            self.scheduler_start_sub(module.name, page_instance.name)

                key1 = f'{module.name}_db_auto_delete'
                key2 = f'{module.name}_db_delete_day'
                if self.P.ModelSetting.has_key(key1) and self.P.ModelSetting.has_key(key2) and self.P.ModelSetting.get_bool(key1):
                    try: module.db_delete(self.P.ModelSetting.get_int(key2))
                    except: pass
                if module.page_list == None:
                    continue
                for page_instance in module.page_list:
                    key1 = f'{module.name}_{page_instance.name}_db_auto_delete'
                    key2 = f'{module.name}_{page_instance.name}_db_delete_day'
                    if self.P.ModelSetting.has_key(key1) and self.P.ModelSetting.has_key(key2) and self.P.ModelSetting.get_bool(key1):
                        try: page_instance.db_delete(self.P.ModelSetting.get_int(key2))
                        except: pass

    def plugin_load_celery(self):
        self.P.logger.debug('%s plugin_load_celery', self.P.package_name)
        for module in self.P.module_list:
            module.plugin_load_celery()
            if module.page_list is not None:
                for page_instance in module.page_list:
                    page_instance.plugin_load_celery()



    def db_init(self):
        try:
            if self.P.ModelSetting is None:
                return

            for key, value in Logic.db_default.items():
                if self.P.ModelSetting.get(key) == None:
                    self.P.ModelSetting.set(key, value)

            for module in self.P.module_list:
                if module.page_list is not None:
                    for page_instance in module.page_list:
                        if page_instance.db_default is not None:
                            for key, value in page_instance.db_default.items():
                                if self.P.ModelSetting.get(key) == None:
                                    self.P.ModelSetting.set(key, value)
                if module.db_default is not None:
                    for key, value in module.db_default.items():
                        if self.P.ModelSetting.get(key) == None:
                            self.P.ModelSetting.set(key, value)
        except Exception as e:
            self.P.logger.error(f'Exception:{str(e)}')
            self.P.logger.error(traceback.format_exc())


    def plugin_unload(self):
        try:
            self.P.logger.debug('%s plugin_unload', self.P.package_name)
            for module in self.P.module_list:
                try:
                    module.plugin_unload()
                    if module.page_list is not None:
                        for page_instance in module.page_list:
                            page_instance.plugin_unload()
                except Exception as e:
                    self.P.logger.error(f'Exception:{str(e)}')
                    self.P.logger.error(traceback.format_exc())
            self.P.logger.debug(f'[{self.P.package_name}] plugin_unload end')
        except Exception as e:
            self.P.logger.error(f'Exception:{str(e)}')
            self.P.logger.error(traceback.format_exc())


    def scheduler_start(self, module_name):
        try:
            job_id = '%s_%s' % (self.P.package_name, module_name)
            module = self.get_module(module_name)
            job = Job(self.P.package_name, job_id, module.get_scheduler_interval(), self.scheduler_function, module.get_scheduler_desc(), args=(module_name,))
            F.scheduler.add_job_instance(job)
        except Exception as e: 
            self.P.logger.error(f'Exception:{str(e)}')
            self.P.logger.error(traceback.format_exc())

    
    def scheduler_stop(self, module_name):
        try:
            job_id = '%s_%s' % (self.P.package_name, module_name)
            F.scheduler.remove_job(job_id)
        except Exception as e: 
            self.P.logger.error(f'Exception:{str(e)}')
            self.P.logger.error(traceback.format_exc())


    def scheduler_function(self, module_name):
        try:
            module = self.get_module(module_name)
            module.scheduler_function()
        except Exception as e: 
            self.P.logger.error(f'Exception:{str(e)}')
            self.P.logger.error(traceback.format_exc())

   
    def db_delete(self, module_name, page_name, day):
        try:
            module = self.get_module(module_name)
            if module == None:
                return False
            if page_name != None:
                page = module.get_page(page_name)
                if page != None:
                    return page.db_delete(day)
            else:
                return module.db_delete(day)
        except Exception as e: 
            self.P.logger.error(f'Exception:{str(e)}')
            self.P.logger.error(traceback.format_exc())


    

    def one_execute(self, module_name):
        self.P.logger.debug('one_execute :%s', module_name)
        try:
            job_id = '%s_%s' % (self.P.package_name, module_name)
            if F.scheduler.is_include(job_id):
                if F.scheduler.is_running(job_id):
                    ret = 'is_running'
                else:
                    F.scheduler.execute_job(job_id)
                    ret = 'scheduler'
            else:
                def func():
                    time.sleep(2)
                    self.scheduler_function(module_name)
                threading.Thread(target=func, args=()).start()
                ret = 'thread'
        except Exception as e: 
            self.P.logger.error(f'Exception:{str(e)}')
            self.P.logger.error(traceback.format_exc())
            ret = 'fail'
        return ret
    
    def immediately_execute(self, module_name):
        self.P.logger.debug('immediately_execute :%s', module_name)
        try:
            def func():
                time.sleep(1)
                self.scheduler_function(module_name)
            threading.Thread(target=func, args=()).start()
            ret = {'ret':'success', 'msg':'실행합니다.'}
        except Exception as e: 
            self.P.logger.error(f'Exception:{str(e)}')
            self.P.logger.error(traceback.format_exc())
            ret = {'ret' : 'danger', 'msg':str(e)}
        return ret

    def get_module(self, sub):
        try:
            for module in self.P.module_list:
                if module.name == sub:
                    return module
        except Exception as e:
            self.P.logger.error(f'Exception:{str(e)}')
            self.P.logger.error(traceback.format_exc())

    """
    def process_telegram_data(self, data, target=None):
        try:
            for module in self.P.module_list:
                if target is None or target.startswith(module.name):
                    module.process_telegram_data(data, target=target)
        except Exception as e:
            self.P.logger.error(f'Exception:{str(e)}')
            self.P.logger.error(traceback.format_exc())
    """



























































    #######################################################
    # 플러그인 - 모듈 - 페이지  구조하에서 서브 관련 함수

    def scheduler_start_sub(self, module_name, page_name):
        try:
            #self.P.logger.warning('scheduler_start_sub')
            job_id = f'{self.P.package_name}_{module_name}_{page_name}'
            ins_module = self.get_module(module_name)
            ins_page = ins_module.get_page(page_name)
            job = Job(self.P.package_name, job_id, ins_page.get_scheduler_interval(), ins_page.scheduler_function, ins_page.get_scheduler_desc(), args=None)
            F.scheduler.add_job_instance(job)
        except Exception as e: 
            self.P.logger.error(f'Exception:{str(e)}')
            self.P.logger.error(traceback.format_exc())

    def scheduler_stop_sub(self, module_name, page_name):
        try:
            job_id = f'{self.P.package_name}_{module_name}_{page_name}'
            F.scheduler.remove_job(job_id)
        except Exception as e: 
            self.P.logger.error(f'Exception:{str(e)}')
            self.P.logger.error(traceback.format_exc())

    def scheduler_function_sub(self, module_name, page_name):
        try:
            ins_module = self.get_module(module_name)
            ins_sub = ins_module.get_page(page_name)
            ins_sub.scheduler_function()
        except Exception as e: 
            self.P.logger.error(f'Exception:{str(e)}')
            self.P.logger.error(traceback.format_exc())

    def one_execute_sub(self, module_name, page_name):
        try:
            job_id = f'{self.P.package_name}_{module_name}_{page_name}'
            if F.scheduler.is_include(job_id):
                if F.scheduler.is_running(job_id):
                    ret = 'is_running'
                else:
                    F.scheduler.execute_job(job_id)
                    ret = 'scheduler'
            else:
                def func():
                    time.sleep(2)
                    self.scheduler_function_sub(module_name, page_name)
                threading.Thread(target=func, args=()).start()
                ret = 'thread'
        except Exception as e: 
            self.P.logger.error(f'Exception:{str(e)}')
            self.P.logger.error(traceback.format_exc())
            ret = 'fail'
        return ret
    
    def immediately_execute_sub(self, module_name, page_name):
        self.P.logger.debug(f'immediately_execute : {module_name} {page_name}')
        try:
            def func():
                time.sleep(1)
                self.scheduler_function_sub(module_name, page_name)
            threading.Thread(target=func, args=()).start()
            ret = {'ret':'success', 'msg':'실행합니다.'}
        except Exception as e: 
            self.P.logger.error(f'Exception:{str(e)}')
            self.P.logger.error(traceback.format_exc())
            ret = {'ret' : 'danger', 'msg':str(e)}
        return ret


    def arg_to_dict(self, arg):
        """
        import urllib.parse

        tmp = urllib.parse.unquote(arg)
        tmps = tmp.split('&')
        ret = {}
        for tmp in tmps:
            _ = tmp.split('=', 1)
            ret[_[0]] = _[1]
        return ret
        """
        import html
        import urllib.parse
        char = '||!||'
        arg = arg.replace('&amp;', char)
        tmp = html.unescape(arg)
        tmp = urllib.parse.unquote(tmp)
        tmp = dict(urllib.parse.parse_qs(tmp, keep_blank_values=True))
        ret = {k: v[0].replace(char, '&') for k, v in tmp.items()}
        return ret



import threading
import traceback
from datetime import datetime, timedelta
from random import randint

from apscheduler.jobstores.base import JobLookupError
from apscheduler.triggers.cron import CronTrigger
from pytz import timezone


class Scheduler(object):
    job_list = []
    first_run_check_thread = None
    def __init__(self, frame):
        self.frame = frame
        self.logger = frame.logger
        try:
            if frame.config['use_gevent']:
                from apscheduler.schedulers.gevent import GeventScheduler
                self.sched = GeventScheduler(timezone='Asia/Seoul')
            else:
                raise Exception('')
        except:
            from apscheduler.schedulers.background import BackgroundScheduler
            self.sched = BackgroundScheduler(timezone='Asia/Seoul')
        self.sched.start()
        self.logger.info('SCHEDULER start..')

    def first_run_check_thread_function(self):
        try:
            #time.sleep(60)
            #for i in range(5):
            flag_exit = True
            for job_instance in self.job_list:
                if not job_instance.run:
                    continue
                if job_instance.count == 0 and not job_instance.is_running and job_instance.is_interval:
                #if job_instance.count == 0 and not job_instance.is_running:
                    job = self.sched.get_job(job_instance.job_id) 
                    if job is not None:
                        self.logger.warning('job_instance : %s', job_instance.plugin)
                        self.logger.warning('XX job re-sched:%s', job)
                        flag_exit = False
                        tmp = randint(1, 20)
                        job.modify(next_run_time=datetime.now(timezone('Asia/Seoul')) + timedelta(seconds=tmp))
                        #break
                    else:
                        pass
            if flag_exit:
                self.remove_job("scheduler_check")
            #time.sleep(30)
        except Exception as e: 
            self.logger.error(f"Exception:{str(e)}")
            self.logger.error(traceback.format_exc())
    
    def shutdown(self):
        self.sched.shutdown()

    def kill_scheduler(self, job_id):
        try:
            self.sched.remove_job(job_id)
        except JobLookupError as err:
            self.logger.debug("fail to stop Scheduler: {err}".format(err=err))
            self.logger.debug(traceback.format_exc())
        

    
    def add_job_instance(self, job_instance, run=True):
        if self.frame.config['run_flask']:
            if not self.is_include(job_instance.job_id):
                job_instance.run = run
                Scheduler.job_list.append(job_instance)
                if job_instance.is_interval:
                    self.sched.add_job(job_instance.job_function, 'interval', minutes=job_instance.interval, seconds=job_instance.interval_seconds, id=job_instance.job_id, args=(None))
                elif job_instance.is_cron:
                    self.sched.add_job(job_instance.job_function, CronTrigger.from_crontab(job_instance.interval), id=job_instance.job_id, args=(None))
                job = self.sched.get_job(job_instance.job_id) 
                if run and job_instance.is_interval:
                    tmp = randint(5, 20)
                    job.modify(next_run_time=datetime.now(timezone('Asia/Seoul')) + timedelta(seconds=tmp))

    def execute_job(self, job_id):
        self.logger.debug('execute_job:%s', job_id)
        job = self.sched.get_job(job_id) 
        tmp = randint(5, 20)
        job.modify(next_run_time=datetime.now(timezone('Asia/Seoul')) + timedelta(seconds=tmp))
         
    
    def is_include(self, job_id):
        job = self.sched.get_job(job_id) 
        return (job is not None)
    
    def remove_job(self, job_id):
        try:
            if self.is_include(job_id):
                self.sched.remove_job(job_id)
                job = self.get_job_instance(job_id)
                if not job.is_running:
                    self.remove_job_instance(job_id)
                self.logger.debug('remove job_id:%s', job_id)
            return True
        except JobLookupError as err:
            self.logger.debug("fail to remove Scheduler: {err}".format(err=err))
            self.logger.debug(traceback.format_exc())
            return False

     
    def get_job_instance(self, job_id):
        for job in Scheduler.job_list:
            if job.job_id == job_id:
                return job
    
    def is_running(self, job_id):
        job = self.get_job_instance(job_id)
        if job is None:
            return False
        else:
            return job.is_running

    # job에서만 호출한다..
    def remove_job_instance(self, job_id):
        # function이 실행중일때 제거하면..
        # 실행중이나 목록에서 빠져버린다..
        for job in Scheduler.job_list:
            if job.job_id == job_id:
                Scheduler.job_list.remove(job)
                self.logger.debug('remove_job_instance : %s', job_id)
                break


    def get_job_list_info(self):
        
        ret = []
        idx = 0
        job_list = self.sched.get_jobs()
        #logger.debug('len jobs %s %s', len(jobs), len(Scheduler.job_list))
        for j in job_list:
            idx += 1
            entity = {}
            entity['no'] = idx
            entity['id'] = j.id
            entity['next_run_time'] = j.next_run_time.strftime('%m-%d %H:%M:%S')
            remain = (j.next_run_time - datetime.now(timezone('Asia/Seoul')))
            tmp = ''
            if remain.days > 0:
                tmp += '%s일 ' % (remain.days)
            remain = remain.seconds
            if remain//3600 > 0:
                tmp += '%s시간 ' % (remain//3600)
            remain = remain % 3600
            if remain // 60 > 0:
                tmp += '%s분 ' % (remain//60)
            tmp += '%s초' % (remain%60)
            #entity['remain_time'] = (j.next_run_time - datetime.now(timezone('Asia/Seoul'))).seconds
            entity['remain_time'] = tmp
            job = self.get_job_instance(j.id)
            if job is not None:
                entity['count'] = job.count
                entity['plugin'] = job.plugin
                if job.is_cron:
                    entity['interval'] = job.interval
                elif job.interval == 9999:
                    entity['interval'] = '항상 실행'
                    entity['remain_time'] = ''
                else:
                    entity['interval'] = '%s분 %s초' % (job.interval, job.interval_seconds)
                entity['is_running'] = job.is_running
                entity['description'] = job.description
                entity['running_timedelta'] = job.running_timedelta.seconds if job.running_timedelta is not None else '-'
                if entity['running_timedelta'] != '-':
                    tmp = entity['running_timedelta']
                    _min = entity['running_timedelta'] / 60
                    _sec = entity['running_timedelta'] % 60
                    entity['running_timedelta'] = "%2d분 %2d초" % (_min, _sec)
                entity['make_time'] = job.make_time.strftime('%m-%d %H:%M:%S')
                entity['run'] = job.run
            else:
                entity['count'] = ''
                entity['plugin'] = ''
                entity['interval'] = ''
                entity['is_running'] = ''
                entity['description'] = ''
                entity['running_timedelta'] = ''
                entity['make_time'] = ''
                entity['run'] = True
            
            ret.append(entity)
        return ret







class Job(object):
    def __init__(self, plugin, job_id, interval, target_function, description, args=None):
        self.plugin = plugin
        self.job_id = job_id
        self.interval = '%s' % interval
        self.interval_seconds = randint(1, 59)
        self.target_function = target_function
        self.description = description
        self.is_running = False
        self.thread = None
        self.start_time = None
        self.end_time = None
        self.running_timedelta = None
        self.status = None
        self.count = 0
        self.make_time = datetime.now(timezone('Asia/Seoul'))
        if len(self.interval.strip().split(' ')) == 5:
            self.is_cron = True
            self.is_interval = False
        else:
            self.is_cron = False
            self.is_interval = True
        if self.is_interval:
            if isinstance(self.interval, str):
                self.interval = int(self.interval)
        self.args = args
        # true이고 interval이면 바로 실행
        # false이면 스케쥴링시간이 되면 실행
        # add_job_instance에서 true이면 20초 이내에 실행하려고 함.
        # false이면 넣을 때는 실행하지 않고 다음 주기때 실행
        self.run = True

    def job_function(self):
        try:
            from framework import F
            self.is_running = True
            self.start_time = datetime.now(timezone('Asia/Seoul'))
            if self.args is None:
                self.thread = threading.Thread(target=self.target_function, args=())
            else:
                self.thread = threading.Thread(target=self.target_function, args=self.args)
            self.thread.daemon = True
            self.thread.start()
            F.socketio.emit('notify', {'type':'success', 'msg':f"{self.description}<br>작업을 시작합니다." }, namespace='/framework')
            self.thread.join()
            F.socketio.emit('notify', {'type':'success', 'msg':f"{self.description}<br>작업이 종료되었습니다." }, namespace='/framework')
            self.end_time = datetime.now(timezone('Asia/Seoul'))
            self.running_timedelta = self.end_time - self.start_time
            self.status = 'success'
            if not F.scheduler.is_include(self.job_id):
                F.scheduler.remove_job_instance(self.job_id)
            self.count += 1
        except Exception as e: 
            self.status = 'exception'
            F.logger.error(f"Exception:{str(e)}")
            F.logger.error(traceback.format_exc())
        finally:
            self.is_running = False
            

        




import random
import string
import time

from support import (SupportDiscord, SupportFile, SupportSlack,
                     SupportSubprocess, SupportTelegram, SupportYaml)
from tool import ToolModalCommand

from .setup import *

name = 'setting'

class ModuleSetting(PluginModuleBase):
    db_default = {
        'db_version' : '1.1',
        'port' : '9999',
        'ddns' : 'http://localhost:9999',
        'use_login' : 'False',
        'web_id': 'admin',
        'web_pw': '21232f297a57a5a743894a0e4a801fc3',
        'web_title': 'Home',
        'use_apikey': 'False',
        'apikey': ''.join(random.choice(string.ascii_uppercase + string.digits) for _ in range(10)),
        'app_id': ''.join(random.choice(string.ascii_uppercase + string.digits) for _ in range(10)),
        f'restart_interval': f'{random.randint(0,59)} {random.randint(1,23)} * * *',
        'restart_notify': 'False',
        'theme' : 'Cerulean',
        'log_level' : '20',
        'system_start_time': '',
        # notify
        'notify_telegram_use' : 'False',
        'notify_telegram_token' : '',
        'notify_telegram_chat_id' : '',
        'notify_telegram_disable_notification' : 'False',
        'notify_discord_use' : 'False',
        'notify_discord_webhook' : '',
        'notify_slack_use' : 'False',
        'notify_slack_webhook' : '',
        'notify_advaned_use' : 'False',
        'notify.yaml': '', #직접 사용하지 않으나 저장 편의상.
        'command_text': '',
        'celery_start_by_web': 'True', #if F.config['running_type'] == 'docker_alpine' else 'False', #웹 실행시 celery 실행
        'celery_start_command': f"celery -A flaskfarm.main.celery worker --loglevel=info --pool=gevent --concurrency=2 --config_filepath={F.config['config_filepath']} --running_type={F.config['running_type']}",
        

    } 

    def __init__(self, P):
        super(ModuleSetting, self).__init__(P, name=name, first_menu='basic')
        

    def process_menu(self, page, req):
        arg = P.ModelSetting.to_dict()
        try:
            if page == 'config':
                arg['config.yaml'] = SupportFile.read_file(F.config['config_filepath'])
                arg['config_filepath'] = F.config['config_filepath']
            elif page == 'export':
                arg['export_filepath'] = F.config['export_filepath']
                if F.config['exist_export']:
                    arg['export.sh'] = SupportFile.read_file(export)
                else:
                    arg['export.sh'] = "export.sh 파일이 없습니다."
            elif page == 'menu':
                arg['menu_yaml_filepath'] = F.config['menu_yaml_filepath']
                arg['menu.yaml'] = SupportFile.read_file(arg['menu_yaml_filepath'])
            elif page == 'notify':
                arg['notify_yaml_filepath'] = F.config['notify_yaml_filepath']
                arg['notify.yaml'] = SupportFile.read_file(arg['notify_yaml_filepath'])
            elif page == 'celery':
                arg['use_celery'] = F.config['use_celery']
                arg['running_type'] = F.config['running_type']
            return render_template(f'{__package__}_{name}_{page}.html', arg=arg)
        except Exception as e: 
            P.logger.error(f'Exception:{str(e)}')
            P.logger.error(traceback.format_exc())
            return render_template('sample.html', title=f"{__package__}/{name}/{page}")


    def process_command(self, command, arg1, arg2, arg3, req):
        ret = {'ret':'success'}
        if command == 'apikey_generate':
            return jsonify(''.join(random.choice(string.ascii_uppercase + string.digits) for _ in range(10)))
        elif command == 'config_save':
            SupportFile.write_file(F.config['config_filepath'], arg1 )
            ret['msg'] = '저장하였습니다.'
        elif command == 'export_save':
            if F.config['exist_export']:
                SupportFile.write_file(F.config['export_filepath'], arg1 )
                ret['msg'] = '저장하였습니다.'
            else:
                ret['ret'] = 'warning'
                ret['msg'] = 'export.sh 파일이 없습니다.'
        elif command == 'menu_save':
            SupportFile.write_file(F.config['menu_yaml_filepath'], arg1 )
            try:
                SupportYaml.read_yaml(F.config['menu_yaml_filepath'])
                ret['msg'] = '저장하였습니다.'
                from framework.init_menu import MenuManager
                MenuManager.init_menu()
                F.socketio.emit("refresh", {}, namespace='/framework')
            except:
                ret['ret'] = "danger" 
                ret['msg'] = "yaml 형식에 맞지 않습니다"
        elif command == 'notify_test':
            if arg1 == 'telegram':
                token, chatid, sound, text = arg2.split('||')
                sound = True if sound == 'true' else False
                SupportTelegram.send_telegram_message(text, image_url=None, bot_token=token, chat_id=chatid, disable_notification=sound)
                ret['msg'] = '메시지를 전송했습니다.'
            elif arg1 == 'discord':
                SupportDiscord.send_discord_message(arg3, webhook_url=arg2)
                ret['msg'] = '메시지를 전송했습니다.'
            elif arg1 == 'slack':
                SupportSlack.send_slack_message(arg3, webhook_url=arg2)
                ret['msg'] = '메시지를 전송했습니다.'
            elif arg1 == 'advanced':
                from tool import ToolNotify
                ToolNotify.send_advanced_message(arg3, message_id=arg2)
                ret['msg'] = '메시지를 전송했습니다.'
        elif command == 'ddns_test':
            try:
                import requests
                url = arg1 + '/version'
                res = requests.get(url)
                data = res.text
                ret['msg'] = f"버전: {data}"
            except Exception as e: 
                P.logger.error(f'Exception:{str(e)}')
                P.logger.error(traceback.format_exc())
                ret['msg'] = str(e)
                ret['type'] = 'warning'
        elif command == 'command_run':
            ret['msg'] = arg1
            pass
        elif command == 'celery_execute':
            self.celery_execute(arg1, mode='foreground')
        elif command == 'celery_execute_back':
            self.celery_execute(arg1, mode='background')
            ret['msg'] = '실행했습니다.'
        elif command == 'celery_test':
            return self.__celery_test()
  
        return jsonify(ret)


    def plugin_load(self):
        try:
            if F.config['run_flask'] == False:
                return

            if F.config['arg_repeat'] == 0 or SystemModelSetting.get('system_start_time') == '':
                SystemModelSetting.set('system_start_time', datetime.now().strftime('%Y-%m-%d %H:%M:%S'))
            SystemModelSetting.set('repeat', str(F.config['arg_repeat']))
            username = SystemModelSetting.get('web_id')
            passwd = SystemModelSetting.get('web_pw')
            F.users[username] = User(username, passwd_hash=passwd)

            self.__set_restart_scheduler()
            self.__set_scheduler_check_scheduler()
            F.get_recent_version()

            notify_yaml_filepath = os.path.join(F.config['path_data'], 'db', 'notify.yaml')
            if os.path.exists(notify_yaml_filepath) == False:
                import shutil
                shutil.copy(
                    os.path.join(F.config['path_app'], 'files', 'notify.yaml.template'),
                    notify_yaml_filepath
                )
            if SystemModelSetting.get_bool('restart_notify'):
                from tool import ToolNotify
                msg = f"시스템이 시작되었습니다.\n재시작: {F.config['arg_repeat']}"
                ToolNotify.send_message(msg, message_id='system_start')
            if SystemModelSetting.get_bool('celery_start_by_web'):
                # 2022-11-14 DB는 flask가 만드는데 만들기전 celery를 실행해버림
                from threading import Timer
                Timer(10, self.celery_execute).start()
                #self.celery_execute()            

        except Exception as e:
            P.logger.error(f'Exception:{str(e)}')
            P.logger.error(traceback.format_exc())

    def plugin_unload(self):
        ToolModalCommand.modal_close()

    def setting_save_after(self, change_list):
        if 'theme' in change_list or 'web_title' in change_list:
            F.socketio.emit("refresh", {}, namespace='/framework')
        elif 'notify.yaml' in change_list:
            try:
                SupportFile.write_file(F.config['notify_yaml_filepath'], SystemModelSetting.get('notify.yaml'))
                SupportYaml.read_yaml(F.config['notify_yaml_filepath'])
            except:
                data = {'type':'danger', 'msg' : "알림 정책이 yaml 형식에 맞지 않습니다."}
                F.socketio.emit("notify", data, namespace='/framework')
        elif 'web_pw' in change_list:
            import hashlib
            enc = hashlib.md5()
            enc.update(SystemModelSetting.get('web_pw').encode())
            SystemModelSetting.set('web_pw', enc.hexdigest())
        elif 'restart_interval' in change_list:
            if SystemModelSetting.get('restart_interval') == '':
                SystemModelSetting.set('restart_interval', '0')
            self.__set_restart_scheduler()
        elif 'log_level' in change_list:
            F.set_level(SystemModelSetting.get_int('log_level'))



    def __set_restart_scheduler(self):
        name = f'{__package__}_restart'
        if F.scheduler.is_include(name):
            F.scheduler.remove_job(name)
        interval = SystemModelSetting.get('restart_interval')
        if interval != '0':
            if len(interval.split(' ')) == 1:
                interval = '%s' % (int(interval) * 60)
            job_instance = Job(__package__, name, interval, F.restart, "자동 재시작")
            F.scheduler.add_job_instance(job_instance, run=False)


    def __set_scheduler_check_scheduler(self):
        name = 'scheduler_check'
        if F.scheduler.is_include(name):
            F.scheduler.remove_job(name)

        job_instance = Job(__package__, name, 2, F.scheduler.first_run_check_thread_function, "Scheduler Check")
        scheduler.add_job_instance(job_instance, run=False)


    def celery_execute(self, command=None, mode='background'):
        if command == None:
            command = SystemModelSetting.get('celery_start_command')
        command = command.replace("{F.config['config_filepath']}", F.config['config_filepath']).replace('{F.config["config_filepath"]}', F.config['config_filepath']).replace("{F.config['path_working']}", F.config['path_working']).replace('{F.config["path_working"]}', F.config['path_working'])
        command = command.split(' ')
        if mode == 'foreground':
            cmd = [
                ['msg', f"명령 : {' '.join(command)}"],
                command,
            ]
            ToolModalCommand.start("Celery 실행", cmd)
        elif mode == 'background':
            SupportSubprocess(command).start(join=False)


    def __celery_test(self):
        if F.config['use_celery']:
            from celery import Celery
            from celery.exceptions import NotRegistered, TimeoutError
            data = {}
            try:
                result = self.celery_test.apply_async()
                try:
                    tmp = result.get(timeout=5, propagate=True)
                except Exception as e:
                    P.logger.error(f'Exception:{str(e)}')
                    P.logger.error(traceback.format_exc())
                data['ret'] = 'success'
                data['msg'] = tmp
            except TimeoutError:
                data['ret'] = 'danger'
                data['msg'] = 'celery가 동작중이 아니거나 모든 Worker가 작업중입니다.'
            except NotRegistered:
                data['ret'] = 'danger'
                data['msg'] = 'Not Registered'
            except:
                data['ret'] = 'danger'
                data['msg'] = '실패'
        else:
            data['ret'] = 'danger'
            data['msg'] = 'celery 실행환경이 아닙니다.'
        P.logger.debug(d(data))
        return data

    #@staticmethod
    @celery.task
    def celery_test():
        try:
            time.sleep(1)
            data = '정상입니다. 이 메시지는 celery 에서 반환됩니다. '
            P.logger.info(data)
            return data
        except Exception as e:
            P.logger.error(f'Exception:{str(e)}')
            P.logger.error(traceback.format_exc())



import os
import platform

from . import logger


class SupportOSCommand(object):

    @classmethod
    def get_size(cls, path):
        from support import SupportFile, SupportSubprocess, SupportUtil
        if platform.system() == 'Windows':
            #https://docs.microsoft.com/en-us/sysinternals/downloads/du
            """
            bin = r'C:\SJVA3\data\bin\du64.exe'
            command = [bin, '-c', '-nobanner', f'"{path}"']
            data = ToolSubprocess.execute_command_return(command, force_log=True)
            logger.warning(data)
            ret = {}
            tmp = data.split('\t')
            ret['target'] = tmp[1].strip()
            ret['size'] = int(tmp[0].strip())
            ret['sizeh'] = ToolUtil.sizeof_fmt(ret['size'])
            """
            ret = {}
            ret['target'] = path
            if os.path.exists(path):
                if os.path.isdir(path):
                    ret['size'] = SupportFile.size(start_path=path)
                else:
                    ret['size'] = os.stat(path).st_size
            ret['sizeh'] = SupportUtil.sizeof_fmt(ret['size'])
            return ret

        else:
            command = ['du', '-bs', path]
            data = SupportSubprocess.execute_command_return(command)
            ret = {}
            tmp = data['log'].split('\t')
            ret['target'] = tmp[1].strip()
            ret['size'] = int(tmp[0].strip())
            ret['sizeh'] = SupportUtil.sizeof_fmt(ret['size'])
        return ret
            


import json
import os
import traceback

from support import SupportSubprocess, d, logger


class SupportRclone(object):
    __instance_list = []

    __rclone_path = 'rclone'
    __rclone_config_path = 'rclone.conf'


    @classmethod
    def initialize(cls, __rclone_path, __rclone_config_path):
        cls.__rclone_path = __rclone_path
        cls.__rclone_config_path = __rclone_config_path

    @classmethod
    def get_rclone_path(cls):
        return cls.__rclone_path

    @classmethod
    def __get_cmd(cls, config_path=None):
        command = [cls.__rclone_path]
        if config_path == None:
            command += ['--config', cls.__rclone_config_path]
        else:
            command += ['--config', config_path]
        return command


    @classmethod
    def rclone_cmd(cls):
        return [cls.__rclone_path, '--config', cls.__rclone_config_path]
        
    @classmethod
    def get_version(cls, rclone_path=None):
        try:
            if rclone_path == None:
                rclone_path = cls.__rclone_path
            cmd = [rclone_path, '--version']
            result = SupportSubprocess.execute_command_return(cmd)
            if result != None and result['status'] == 'finish':
                return result['log']
        except Exception as e: 
            logger.error(f'Exception:{str(e)}')
            logger.error(traceback.format_exc())   


    @classmethod
    def config_list(cls, rclone_path=None, rclone_config_path=None, option=None):
        try:
            if rclone_path == None:
                rclone_path = cls.__rclone_path
            if rclone_config_path == None:
                rclone_config_path = cls.__rclone_config_path
            if os.path.exists(rclone_config_path) == False:
                return
            command = [rclone_path, '--config', rclone_config_path, 'config', 'dump']
            if option is not None:
                command += option
            result = SupportSubprocess.execute_command_return(command, format='json')
            for key, value in result['log'].items():
                if 'token' in value and value['token'].startswith('{'):
                    value['token'] = json.loads(value['token'])
            return result['log']
        except Exception as e: 
            logger.error(f'Exception:{str(e)}')
            logger.error(traceback.format_exc())   

    @classmethod
    def get_config(cls, remote_name, rclone_path=None, rclone_config_path=None, option=None):
        try:
            data = cls.config_list(rclone_path=rclone_path, rclone_config_path=rclone_config_path, option=option)
            return data.get(remote_name, None)
            
        except Exception as e: 
            logger.error(f"Exception:{str(e)}")
            logger.error(traceback.format_exc())


    @classmethod
    def lsjson(cls, remote_path, config_path=None, option=None):
        return cls.__execute_one_param('lsjson', remote_path, config_path=config_path, option=option, format='json')


    @classmethod
    def lsf(cls, remote_path, config_path=None, option=None):
        if option == None:
            option = ['--max-depth=1']
        return cls.__execute_one_param('lsf', remote_path, config_path=config_path, option=option, format='json')


    @classmethod
    def size(cls, remote_path, config_path=None, option=None):
        if option == None:
            option = ['--json']
        return cls.__execute_one_param('size', remote_path, config_path=config_path, option=option, format='json')


    @classmethod
    def mkdir(cls, remote_path, config_path=None, option=None):
        return cls.__execute_one_param('mkdir', remote_path, config_path=config_path, option=option, format='json')

    @classmethod
    def purge(cls, remote_path, config_path=None, option=None):
        return cls.__execute_one_param('purge', remote_path, config_path=config_path, option=option, format='json')


    @classmethod
    def __execute_one_param(cls, command, remote_path, config_path=None, option=None, format=None):
        try:
            command = cls.__get_cmd(config_path) + [command, remote_path]
            if option is not None:
                command += option
            result = SupportSubprocess.execute_command_return(command, format=format)
            ret = None
            if result != None and result['status'] == 'finish':
                ret = result['log']
            return ret
        except Exception as e: 
            logger.error(f'Exception:{str(e)}')
            logger.error(traceback.format_exc())  


    @classmethod
    def copy(cls, src, tar, config_path=None, option=None):
        return cls.__execute_two_param('copy', src, tar, config_path=config_path, option=option)
    
    @classmethod
    def copy_server_side(cls, src, tar, config_path=None, option=None):
        if option == None:
            option = ['--drive-server-side-across-configs=true', '--delete-empty-src-dirs']
        return cls.__execute_two_param('copy', src, tar, config_path=config_path, option=option)

    @classmethod
    def move(cls, src, tar, config_path=None, option=None):
        return cls.__execute_two_param('move', src, tar, config_path=config_path, option=option)

    @classmethod
    def move_server_side(cls, src, tar, config_path=None, option=None):
        if option == None:
            option = ['--drive-server-side-across-configs=true', '--delete-empty-src-dirs']
        return cls.__execute_two_param('move', src, tar, config_path=config_path, option=option)

    @classmethod
    def __execute_two_param(cls, command, src, tar, config_path=None, option=None, format=None):
        try:
            command = cls.__get_cmd(config_path) + [command, src, tar]
            if option is not None:
                command += option
            result = SupportSubprocess.execute_command_return(command, format=format)
            ret = None
            if result != None and result['status'] == 'finish':
                ret = result['log']
            return ret
        except Exception as e: 
            logger.error(f'Exception:{str(e)}')
            logger.error(traceback.format_exc())  



    @classmethod
    def getid(cls, remote_path, config_path=None, option=None):
        try:
            command = cls.__get_cmd(config_path) + ['backend', 'getid', remote_path]
            if option is not None:
                command += option
            result = SupportSubprocess.execute_command_return(command)
            ret = None
            if result != None and result['status'] == 'finish':
                ret = result['log']
            if ret is not None and (len(ret.split(' ')) > 1 or ret == ''):
                ret = None
            return ret
        except Exception as e: 
            logger.error(f"Exception:{str(e)}")
            logger.error(traceback.format_exc())


    @classmethod
    def chpar(cls, src, tar, config_path=None, option=None):
        try:
            command = cls.__get_cmd(config_path) + ['backend', 'chpar', src, tar, '-o', 'depth=1', '-o', 'delete-empty-src-dir', '--drive-use-trash=false']
            if option is not None:
                command += option
            result = SupportSubprocess.execute_command_return(command)
            ret = None
            if result != None and result['status'] == 'finish':
                ret = result['log']
            return True
        except Exception as e: 
            logger.error(f"Exception:{str(e)}")
            logger.error(traceback.format_exc())
        return False


import sqlite3
import traceback
from datetime import datetime, timedelta

from framework import F
from sqlalchemy import desc, or_


class ModelBase(F.db.Model):
    __abstract__ = True
    __table_args__ = {'mysql_collate': 'utf8_general_ci'}
    P = None

    def __repr__(self):
        return repr(self.as_dict())

    def as_dict(self):
        return {x.name: getattr(self, x.name).strftime('%m-%d %H:%M:%S') if isinstance(getattr(self, x.name), datetime) else getattr(self, x.name) for x in self.__table__.columns}
    

    def save(self):
        try:
            with F.app.app_context():
                F.db.session.add(self)
                F.db.session.commit()
                return self
        except Exception as e:
            self.P.logger.error(f'Exception:{str(e)}')
            self.P.logger.error(traceback.format_exc())

    @classmethod
    def get_paging_info(cls, count, current_page, page_size):
        try:
            paging = {}
            paging['prev_page'] = True
            paging['next_page'] = True
            if current_page <= 10:
                paging['prev_page'] = False
            
            paging['total_page'] = int(count / page_size) + 1
            if count % page_size == 0:
                paging['total_page'] -= 1
            paging['start_page'] = int((current_page-1)/10) * 10 + 1
            paging['last_page'] = paging['total_page'] if paging['start_page'] + 9 > paging['total_page'] else paging['start_page'] + 9
            if paging['last_page'] == paging['total_page']:
                paging['next_page'] = False
            paging['current_page'] = current_page
            paging['count'] = count
            #F.logger.debug('paging : c:%s %s %s %s %s %s', count, paging['total_page'], paging['prev_page'], paging['next_page'] , paging['start_page'], paging['last_page'])
            return paging
        except Exception as e:
            cls.P.logger.error(f'Exception:{str(e)}')
            cls.P.logger.error(traceback.format_exc())

    
    @classmethod
    def get_by_id(cls, id):
        try:
            with F.app.app_context():
                return F.db.session.query(cls).filter_by(id=int(id)).first()
        except Exception as e:
            cls.P.logger.error(f'Exception:{str(e)}')
            cls.P.logger.error(traceback.format_exc())


    @classmethod
    def get_list(cls, by_dict=False):
        try:
            with F.app.app_context():
                tmp = F.db.session.query(cls).all()
                if by_dict:
                    tmp = [x.as_dict() for x in tmp]
                return tmp
        except Exception as e:
            cls.P.logger.error(f'Exception:{str(e)}')
            cls.P.logger.error(traceback.format_exc())



    @classmethod
    def delete_by_id(cls, id):
        try:
            with F.app.app_context():
                F.db.session.query(cls).filter_by(id=int(id)).delete()
                F.db.session.commit()
                return True
        except Exception as e:
            cls.P.logger.error(f'Exception:{str(e)}')
            cls.P.logger.error(traceback.format_exc())
        return False

    @classmethod
    def delete_all(cls, day=None):
        count = -1
        try:
            with F.app.app_context():
                if day == None or day in [0, '0']:
                    count = F.db.session.query(cls).delete()
                else:
                    now = datetime.now()
                    ago = now - timedelta(days=int(day))
                    #ago.hour = 0
                    #ago.minute = 0
                    count = F.db.session.query(cls).filter(cls.created_time < ago).delete()
                    cls.P.logger.info(f"delete_all {day=} {count=}")
                F.db.session.commit()
               
                db_file = F.app.config['SQLALCHEMY_BINDS'][cls.P.package_name].replace('sqlite:///', '').split('?')[0]
                connection = sqlite3.connect(db_file)
                cursor = connection.cursor()
                cursor.execute('VACUUM;')
                connection.close()
        except Exception as e:
            cls.P.logger.error(f'Exception:{str(e)}')
            cls.P.logger.error(traceback.format_exc())
        return count
    

    @classmethod
    def web_list(cls, req):
        try:
            ret = {}
            page = 1
            page_size = 30
            search = ''
            if 'page' in req.form:
                page = int(req.form['page'])
            if 'keyword' in req.form:
                search = req.form['keyword'].strip()
            option1 = req.form.get('option1', 'all')
            option2 = req.form.get('option2', 'all')
            order = req.form['order'] if 'order' in req.form else 'desc'

            query = cls.make_query(req, order=order, search=search, option1=option1, option2=option2)
            count = query.count()
            query = query.limit(page_size).offset((page-1)*page_size)
            #F.logger.debug('cls count:%s', count)
            lists = query.all()
            ret['list'] = [item.as_dict() for item in lists]
            ret['paging'] = cls.get_paging_info(count, page, page_size)
            try:
                if cls.P.ModelSetting is not None and cls.__tablename__ is not None:
                    cls.P.ModelSetting.set(f'{cls.__tablename__}_last_list_option', f'{order}|{page}|{search}|{option1}|{option2}')
            except Exception as e:
                F.logger.error(f"Exception:{str(e)}")
                F.logger.error(traceback.format_exc())
                F.logger.error(f'{cls.__tablename__}_last_list_option ERROR!' )
            return ret
        except Exception as e:
            cls.P.logger.error(f"Exception:{str(e)}")
            cls.P.logger.error(traceback.format_exc())


    # 오버라이딩
    @classmethod
    def make_query(cls, req, order='desc', search='', option1='all', option2='all'):
        with F.app.app_context():
            query = F.db.session.query(cls)
            if order == 'desc':
                query = query.order_by(desc(cls.id))
            else:
                query = query.order_by(cls.id)
            return query 
        
    
    @classmethod
    def make_query_search(cls, query, search, field):
        if search is not None and search != '':
            if search.find('|') != -1:
                tmp = search.split('|')
                conditions = []
                for tt in tmp:
                    if tt != '':
                        conditions.append(field.like('%'+tt.strip()+'%') )
                query = query.filter(or_(*conditions))
            elif search.find(',') != -1:
                tmp = search.split(',')
                for tt in tmp:
                    if tt != '':
                        query = query.filter(field.like('%'+tt.strip()+'%'))
            else:
                query = query.filter(field.like('%'+search+'%'))
            #query = query1.union(query2)
        return query


    @classmethod
    def get_list_by_status(cls, status):
        try:
            with F.app.app_context():
                query = F.db.session.query(cls).filter(
                    cls.status == status,
                )
                query = query.order_by(cls.id)
                return query.all()
        except:
            pass


import os
import threading
import time
import traceback

from flask import request
from framework import F
from support import SingletonClass

namespace = 'log'

@F.socketio.on('connect', namespace='/%s' % namespace)
def socket_connect(): 
    F.logger.debug('log connect')

@F.socketio.on('start', namespace='/%s' % namespace)
def socket_file(data):
    try:
        package = filename = None
        if 'package' in data:
            package = data['package']
        else:
            filename = data['filename']
        LogViewer.instance().start(package, filename, request.sid)
        F.logger.debug('start package:%s filename:%s sid:%s', package, filename, request.sid)
    except Exception as e: 
        F.logger.error(f"Exception:{str(e)}")
        F.logger.error(traceback.format_exc())

@F.socketio.on('disconnect', namespace='/%s' % namespace)
def disconnect():
    try:
        LogViewer.instance().disconnect(request.sid)
        F.logger.debug('disconnect sid:%s', request.sid)
    except Exception as e: 
        F.logger.error(f"Exception:{str(e)}")
        F.logger.error(traceback.format_exc())



class WatchThread(threading.Thread):

    def __init__(self, package, filename):
        super(WatchThread, self).__init__()
        self.stop_flag = False
        self.package = package
        self.filename = filename
        self.daemon = True

    def stop(self):
        self.stop_flag = True

    def run(self):
        F.logger.debug('WatchThread.. Start %s', self.package)
        if self.package is not None:
            logfile = os.path.join(F.config['path_data'], 'log', f'{self.package}.log')
            key = 'package'
            value = self.package
        else:
            logfile = os.path.join(F.config['path_data'], 'log', self.filename)
            key = 'filename'
            value = self.filename
        if os.path.exists(logfile):
            with open(logfile, 'r', encoding='utf8') as f:
                f.seek(0, os.SEEK_END)
                while not self.stop_flag:
                    line = f.readline()
                    if not line:
                        time.sleep(0.1) # Sleep briefly
                        continue
                    F.socketio.emit("add", {key : value, 'data': line}, namespace='/log')
            F.logger.debug('WatchThread.. End %s', value)
        else:
            F.socketio.emit("add", {key : value, 'data': 'not exist logfile'}, namespace='/log')


class LogViewer(SingletonClass):
           
    watch_list = {} 

    @classmethod
    def start(cls, package, filename, sid):
        # 2019-04-02 간만에 봤더니 헷깔려서 적는다
        # 이 쓰레드는 오픈시 이전 데이타만을 보내는 쓰레드다. 실시간보는거 아님.
        
        def thread_function():
            if package is not None:
                logfile = os.path.join(F.config['path_data'], 'log', f'{package}.log')
            else:
                logfile = os.path.join(F.config['path_data'], 'log', filename)
            if os.path.exists(logfile):
                ins_file = open(logfile, 'r', encoding='utf8')  ## 3)
                line = ins_file.read()
                F.socketio.emit("on_start", {'data': line}, namespace='/log')
                F.logger.debug('on_start end')
            else:
                F.socketio.emit("on_start", {'data': 'not exist logfile'}, namespace='/log')
        
        if package is not None:
            key = package
        else:
            key = filename
        thread = threading.Thread(target=thread_function, args=())
        thread.daemon = True
        thread.start()
      

        if key not in cls.watch_list:
            cls.watch_list[key] = {}
            cls.watch_list[key]['sid'] = []
            cls.watch_list[key]['thread'] = WatchThread(package, filename)
            cls.watch_list[key]['thread'].start()
        cls.watch_list[key]['sid'].append(sid)

    @classmethod
    def disconnect(cls, sid):
        find = False
        find_key = None
        for key, value in cls.watch_list.items():
            F.logger.debug('key:%s value:%s', key, value)
            for s in value['sid']:
                if sid == s:
                    find = True
                    find_key = key
                    value['sid'].remove(s)
                    break
            if find:
                break
        if not find:
            return
        if not cls.watch_list[find_key]['sid']:
            F.logger.debug('thread kill')
            cls.watch_list[find_key]['thread'].stop()
            del cls.watch_list[find_key]




import os, sys, traceback, subprocess, json, platform, time
import shutil
from . import logger


class SupportFfmpeg(object):

    @classmethod
    def download_m3u8(cls, config):
        try:
            
            if config.get('proxy') == None:
                if config.get('headers') == None:
                    command = [config.get('ffmpeg_path'), '-y', '-correct_ts_overflow', '0', '-i', config['url'], '-c', 'copy', '-bsf:a', 'aac_adtstoasc']
                else:
                    headers_command = []
                    for key, value in config.get('headers').items():
                        if key.lower() == 'user-agent':
                            headers_command.append('-user_agent')
                            headers_command.append(value)
                            pass
                        else:
                            headers_command.append('-headers')
                            headers_command.append('\'%s:%s\''%(key,value))
                    command = [config.get('ffmpeg_path'), '-y', '-correct_ts_overflow', '0'] + headers_command + ['-i', config['url'], '-c', 'copy', '-bsf:a', 'aac_adtstoasc']
            else:
                command = [config.get('ffmpeg_path'), '-y', '-correct_ts_overflow', '0', '-http_proxy', config.get('proxy'), '-i', config['url'], '-c', 'copy', '-bsf:a', 'aac_adtstoasc']

            filename = str(int(time.time())) + '.mp4'
            tmp = config.get('tmp_dir')
            if tmp == None:
                tmp = os.getcwd()
            tmp_filepath = os.path.join(tmp, filename)
            command.append(tmp_filepath)
            logger.debug(' '.join(command))
            from . import SupportSubprocess
            ret = SupportSubprocess.execute(command, timeout=10)
            logger.error(ret)
            if os.path.exists(tmp_filepath):
                shutil.move(tmp_filepath, config.get('output_filepath'))


        except Exception as e: 
            logger.error(f'Exception:{str(e)}', )
            logger.error(traceback.format_exc())
            logger.error('command : %s', command)
            


def d(data):
    if type(data) in [type({}), type([])]:
        import json
        try:
            return '\n' + json.dumps(data, indent=4, ensure_ascii=False)
        except:
            return data
    else:
        return str(data)

from .logger import get_logger

#logger = get_logger()
import logging
logger = logging.getLogger('support')

from .base.aes import SupportAES
from .base.discord import SupportDiscord
from .base.file import SupportFile
from .base.os_command import SupportOSCommand
from .base.string import SupportString
from .base.sub_process import SupportSubprocess
from .base.support_sc import SupportSC
from .base.telegram import SupportTelegram
from .base.slack import SupportSlack
from .base.util import (AlchemyEncoder, SingletonClass, SupportUtil,
                        default_headers, pt)
from .base.yaml import SupportYaml



import time
import traceback

import requests
from telepot_mod import Bot

from . import logger


class SupportTelegram:

    @classmethod
    def send_telegram_message(cls, text, bot_token=None, chat_id=None, image_url=None,  disable_notification=None):
        try:
            bot = Bot(bot_token)
            if image_url is not None:
                logger.debug(image_url)
                for i in range(5):
                    if requests.get(image_url).status_code == 200:
                        break
                    else:
                        time.sleep(3)
                try:
                    bot.sendPhoto(chat_id, image_url, disable_notification=disable_notification)
                except Exception as e: 
                    logger.error(f"Exception:{str(e)}")
                    logger.error(traceback.format_exc()) 
            bot.sendMessage(chat_id, text, disable_web_page_preview=True, disable_notification=disable_notification)
            return True
        except Exception as e: 
            logger.error(f"Exception:{str(e)}")
            logger.error(traceback.format_exc()) 
            logger.debug('Chatid:%s', chat_id)
        return False




import re
import time
import traceback
from datetime import datetime

from support import SupportFile, SupportString

from . import logger

EXTENSION = 'mp4|avi|mkv|ts|wmv|m2ts|smi|srt|ass|m4v|flv|asf|mpg|ogm'

REGEXS = [
    r'^(?P<name>.*?)\.([sS](?P<sno>\d+))?[eE](?P<no>\d+)(\-E\d{1,4})?\.?(?P<a>.*?\.)?(?P<date>\d{6})\.(?P<etc>.*?)((?P<quality>\d+)[p|P])?(\-?(?P<release>.*?))?(\.(.*?))?$',
    r'^(?P<name>.*?)\s([sS](?P<sno>\d+))?[eE](?P<no>\d+)(\-E\d{1,4})?\.?(END\.)?(?P<date>\d{6})\.(?P<etc>.*?)(?P<quality>\d+)[p|P](?P<more>\..*?)(?P<ext>\.[\w|\d]{3})$',
    r'^(?P<name>.*?)\.([sS](?P<sno>\d+))?(E(?P<no>\d+)\.?)?(END\.)?(?P<date>\d{6})\.(?P<etc>.*?)(?P<quality>\d+)[p|P](\-?(?P<release>.*?))?(\.(.*?))?$',
    r'^(?P<name>.*?)([sS](?P<sno>\d+))?[eE](?P<no>\d+)', # 외국 릴
    r'^(?P<name>.*?)\.(Series\.(?P<sno>\d+)\.)?(?P<no>\d+)of', # 외국 릴
    r'^(?P<name>.*?)[\s\(](?P<no>\d+)[회화]',
]

#합본처리 제외
#_REGEX_FILENAME_RENAME = r'(?P<title>.*?)[\s\.]E?(?P<no>\d{1,2})[\-\~\s\.]?E?\d{1,2}'

class EntityKtv(object):
    meta_cache = {}
    def __init__(self, filename, dirname=None, meta=False, is_title=False, config=None):
        self.data = {
            'filename' : {
                'original' : filename, 
                'dirname' : dirname, 
                'is_matched' : False,
                'match_index' : -1,
                'name' : '',
                'original_name': '',
            },
            'meta' : {
                'find':False,
            },
            'process_info' : {
                'rebuild':'',
                'status':''
            }
        }
        if is_title == False:
            self.analyze(config=config)
            self.data['filename']['original_name'] = self.data['filename']['name']
            if self.data['filename']['name'] != '' and config is not None:
                rule = config.get('검색어 변경', None)
                if rule is not None:
                    self.change_name(rule)
        else:
            self.data['filename']['name'] = filename
            self.data['filename']['is_matched'] = True
            self.data['filename']['match_index'] = -1
            self.data['filename']['date'] = ''


        search_try = False
        if meta and self.data['filename']['is_matched']:
            if self.data['filename']['match_index'] in [3, 4]:
                
                info = SupportString.language_info(self.data['filename']['name'])
                if info[0] == 0:
                    search_try = True
                    self.find_meta_tmdb()

            if search_try == False:
                self.find_meta()

            if self.data['meta']['find']:
                self.find_meta_season()
                try:
                    #logger.warning(f"찾은 메타 : {self.data['meta']['info']['title']} {self.data['meta']['info']['code']}")
                    if self.data['filename']['date'] == '':
                        self.data['process_info']['status'] = 'no_date'
                    else:
                        self.check_episode_no()
                except Exception as e:
                    logger.debug(f"Exception:{str(e)}")
                    logger.debug(traceback.format_exc())


    def analyze(self, config=None):
        def get(md, field):
            if field in md and md[field] is not None:
                return md[field]
            return ''

        for idx, regex in enumerate(REGEXS):
            match = re.compile(regex).match(self.data['filename']['original'])
            if not match:
                continue
            md = match.groupdict()
            self.data['filename']['is_matched'] = True
            self.data['filename']['match_index'] = idx
            self.data['filename']['name'] = get(md, 'name').replace('.', ' ').strip()
            tmp = get(md, 'sno')
            self.data['filename']['sno'] = int(tmp) if tmp != '' else 1
            tmp = get(md, 'no')
            try:
                self.data['filename']['no'] = int(tmp) if tmp != '' else -1
                if self.data['filename']['no'] == 0:
                    raise Exception('0')
            except:
                self.data['process_info']['rebuild'] += 'remove_episode'
                self.data['filename']['no'] = -1

            self.data['filename']['date'] = get(md, 'date')
            self.data['filename']['etc'] = get(md, 'etc')
            self.data['filename']['quality'] = get(md, 'quality')
            self.data['filename']['release'] = get(md, 'release')
            self.data['filename']['more'] = get(md, 'more')
            self.data['filename']['day_delta'] = 0
            if self.data['filename']['date'] != '':
                today = datetime.now()
                try:
                    tmp = str(self.data['filename']['date'])
                    if tmp[0] in ['8', '9']:
                        tmp = '19' + tmp
                    else:
                        tmp = '20' + tmp
                    max_date = datetime.strptime(tmp, '%Y%m%d')
                except:
                    max_date = today
                self.data['filename']['day_delta'] = (today - max_date).days
            
            
            #logger.warning(d(self.data['filename']))
            break

        if config is not None:
            rule = config.get('에피소드 번호 삭제 목록', [])
            if self.data['filename']['name'] in rule:
                self.data['process_info']['rebuild'] += 'remove_episode_by_rule'
                self.data['filename']['no'] = -1
                
    
    def change_name(self, rules):
        name = self.data['filename']['name']
        for rule in rules:
            try:
                name = re.sub(rule['source'], rule['target'], name, flags=re.I).strip()
            except Exception as e: 
                logger.error(f"Exception:{e}")
                logger.error(traceback.format_exc())
        self.data['filename']['name'] = name



    def check_episode_no(self):
        if self.data['filename']['no'] > 0 and self.data['filename']['no'] in self.data['meta']['info']['extra_info']['episodes']:
            #logger.warning(f"에피소드 정보 있음")
            #logger.warning(self.data['meta']['info']['extra_info']['episodes'][self.data['filename']['no']])
            tmp = self.data['meta']['info']['extra_info']['episodes'][self.data['filename']['no']]
            # daum만 체크
            if 'daum' in tmp:
                value = tmp['daum']
                tmp2 = value['premiered']
                if self.data['filename']['date'] == tmp2.replace('-', '')[2:]:
                    self.data['process_info']['status'] = 'number_and_date_match'
                    self.data['process_info']['episode'] = value
                    self.data['process_info']['episode']['no'] = self.data['filename']['no']
                    return
                else:
                    #하루차이는 매칭시킴
                    if abs(int(self.data['filename']['date']) - int(tmp2.replace('-', '')[2:])) in [1, 70, 71, 72, 73, 8870]:
                        self.data['process_info']['status'] = 'number_and_date_match'
                        self.data['process_info']['rebuild'] += 'change_date'
                        self.data['process_info']['change_date'] = tmp2.replace('-', '')[2:]
                        self.data['process_info']['episode'] = value
                        self.data['process_info']['episode']['no'] = self.data['filename']['no']
                        return

        # 맞는 에피소드 몾찾음
        if len(self.data['meta']['info']['extra_info']['episodes']) == 0:
            # 메타검색은 했지만 에피소드 목록이 없음.
            self.data['process_info']['status'] = 'meta_epi_empty'
            return

        # 방송일에 맞는 에피 번호 찾기
        #logger.warning(f"에피소드 목록")

        for epi_no, value in self.data['meta']['info']['extra_info']['episodes'].items():
            if 'daum' in value:
                site_info = value['daum']
                tmp2 = site_info['premiered']
                if self.data['filename']['date'] == tmp2.replace('-', '')[2:]:
                    self.data['process_info']['status'] = 'number_and_date_match'
                    self.data['process_info']['rebuild'] += 'change_epi_number'
                    self.data['process_info']['change_epi_number'] = epi_no
                    self.data['process_info']['episode'] = value['daum']
                    self.data['process_info']['episode']['no'] = epi_no
                    return
        
        # 다음에서 몾찾았지만 티빙 웨이브에 있다면 그대로 유지해야함.
        # 굳이 찾을 필요없이 릴리즈로 맞다고 넘김
        # 근데 받을때는 에피번호가 없고 나중에 메타가 생기는 경우가 잇는 것 같음
        if self.data['filename']['no'] != -1:
            if self.data['filename']['release'] in ['ST', 'SW', 'SWQ', 'STQ', 'ODK']:
                self.data['process_info']['status'] = 'number_and_date_match_by_release'
                return
        else:
            for epi_no, value in self.data['meta']['info']['extra_info']['episodes'].items():
                for site, site_info in value.items():
                    if site == 'daum':
                        continue
                    tmp2 = site_info['premiered']
                    if self.data['filename']['date'] == tmp2.replace('-', '')[2:]:
                        self.data['process_info']['status'] = 'number_and_date_match_ott'
                        self.data['process_info']['rebuild'] += 'change_epi_number'
                        self.data['process_info']['change_epi_number'] = epi_no
                        self.data['process_info']['episode'] = site_info
                        self.data['process_info']['episode']['no'] = epi_no
                        
                        return


        #logger.error("에피소드 목록이 있지만 맞는 메타를 찾지 못함")
        #logger.error(f"에피소드 번호 {epi_no}")
        #logger.error(f"에피소드 번호 {self.data['filename']['original']}")
        #logger.warning(d(self.data['meta']['info']['extra_info']['episodes']))
        #logger.debug("티빙, 웨이브에서 찾음")

        if self.data['filename']['no'] > 0 and self.data['filename']['no'] in self.data['meta']['info']['extra_info']['episodes']:
            #logger.warning(f"에피소드 정보 있음 22")
            #logger.warning(self.data['meta']['info']['extra_info']['episodes'][self.data['filename']['no']])
            tmp = self.data['meta']['info']['extra_info']['episodes'][self.data['filename']['no']]
            # daum만 체크
            for site, value in tmp.items():
                if site == 'daum':
                    continue
                tmp2 = value['premiered']
                if self.data['filename']['date'] == tmp2.replace('-', '')[2:]:
                    self.data['process_info']['status'] = 'number_and_date_match'
                    self.data['process_info']['episode'] = value
                    self.data['process_info']['episode']['no'] = self.data['filename']['no']
                    return
                else:
                    #하루차이는 매칭시킴
                    if abs(int(self.data['filename']['date']) - int(tmp2.replace('-', '')[2:])) in [1, 70, 71, 72, 73, 8870]:
                        self.data['process_info']['status'] = 'number_and_date_match'
                        self.data['process_info']['rebuild'] += 'change_date'
                        self.data['process_info']['change_date'] = tmp2.replace('-', '')[2:]
                        self.data['process_info']['episode'] = value
                        self.data['process_info']['episode']['no'] = self.data['filename']['no']
                        return

        for epi_no, value in self.data['meta']['info']['extra_info']['episodes'].items():
            if epi_no == 0:
                continue
            for site, site_info in value.items(): 
                if site == 'daum':
                    continue
                tmp2 = site_info['premiered']
                if self.data['filename']['date'] == tmp2.replace('-', '')[2:]:
                    #logger.warning(f"2222 다음에서 새로운 에피소드 번호 찾음 : {epi_no}")
                    #logger.warning(d(site_info))
                    self.data['process_info']['status'] = 'number_and_date_match'
                    self.data['process_info']['rebuild'] += 'change_epi_number'
                    self.data['process_info']['change_epi_number'] = epi_no
                    self.data['process_info']['episode'] = site_info
                    self.data['process_info']['episode']['no'] = epi_no
                    return

        if epi_no < self.data['filename']['no']:
            self.data['process_info']['status'] = 'meta_epi_not_find'
            
        self.data['process_info']['status'] = 'meta_epi_not_find'
        #for tmp in self.data['meta']['info']['extra_info']['episode']:
        #    logger.debug((tmp))




    
    def find_meta(self):
        from support_site import SiteDaumTv, SiteTvingTv, SiteWavveTv
        module_map = [('daum', SiteDaumTv), ('tving',SiteTvingTv), ('wavve',SiteWavveTv)]
        #if self.data['filename']['name'] in EntityKtv.meta_cache:
        #    self.data['meta'] = EntityKtv.meta_cache[self.data['filename']['name']]
        #    return
        #module_list = [SiteDaumTv, SiteTvingTv, SiteWavveTv]
        #module_list = [SiteDaumTv]
        
        for site, site_class in module_map:
            try:
                if self.data['filename']['name'] in EntityKtv.meta_cache and site in EntityKtv.meta_cache[self.data['filename']['name']]:
                    self.data['meta'] = EntityKtv.meta_cache[self.data['filename']['name']][site]
                    # 없는 것도 저장하여 중복검색 방지
                    if self.data['meta']['find']:
                        return
                site_data = site_class.search(self.data['filename']['name'])
                #logger.warning(f"{site} {d(site_data)}")
                if site_data['ret'] == 'success':
                    if site == 'daum':
                        self.data['meta']['search'] = site_data['data']
                        self.data['meta']['info'] = site_class.info(self.data['meta']['search']['code'], self.data['meta']['search']['title'])['data']
                        # Daum이 미국드라마, 일본드라마 등의 외국 드라마 장르이면서 ST SW 릴이면 Daum보다는 OTT 메타를 사용하도록 함
                        if self.data['meta']['info']['genre'][0] != '드라마' and self.data['meta']['info']['genre'][0].find('드라마') != -1 and self.data['filename'].get('release', '') in ['ST', 'SW']:
                            continue

                        SiteTvingTv.apply_tv_by_search(self.data['meta']['info'], force_search_title=self.data['filename']['name'])
                        SiteWavveTv.apply_tv_by_search(self.data['meta']['info'], force_search_title=self.data['filename']['name'])
                        if self.data['meta']['info']['episode'] == -1:
                            self.data['meta']['info']['episode'] = len(self.data['meta']['info']['extra_info']['episodes'].keys())
                        self.data['meta']['find'] = True
                        
                    else:
                        if len(site_data['data']) > 0 and site_data['data'][0]['score'] > 90:
                            self.data['meta']['search'] = site_data['data'][0]
                            self.data['meta']['info'] = site_class.info(self.data['meta']['search']['code'])['data']
                            self.data['meta']['find'] = True
                
                if self.data['meta']['find']:
                    if len(self.data['meta']['info']['genre']) == 0:
                        self.data['meta']['info']['genre'].append('기타')
                    if self.data['filename']['name'] not in EntityKtv.meta_cache:
                        EntityKtv.meta_cache[self.data['filename']['name']] = {}
                    EntityKtv.meta_cache[self.data['filename']['name']][site] = self.data['meta']
                    return
                else:
                    if self.data['filename']['name'] not in EntityKtv.meta_cache:
                        EntityKtv.meta_cache[self.data['filename']['name']] = {}
                    EntityKtv.meta_cache[self.data['filename']['name']][site] = self.data['meta']
            except Exception as e:
                logger.error(f"Exception:{str(e)}")
                logger.error(traceback.format_exc())


    def get_newfilename(self):
        if self.data['filename']['match_index'] == 2:
           self.data['process_info']['rebuild'] += f"match_{self.data['filename']['match_index']}"

        if self.data['process_info']['rebuild'] in ['', 'match_2', 'meta_epi_not_find', 'match_3']:
            return self.data['filename']['original']

        elif self.data['process_info']['rebuild'] == 'remove_episode' or self.data['process_info']['rebuild'].find('remove_episode_by_rule') != -1:
            return re.sub('\.[eE].*?\.', '.', self.data['filename']['original'])
        elif self.data['process_info']['rebuild'] == 'remove_episodechange_epi_number' and self.data['process_info']['change_epi_number'] == 0:
            return re.sub('\.[eE].*?\.', '.', self.data['filename']['original'])   
        
        elif self.data['process_info']['rebuild'] == 'change_epi_number':
            return re.sub('\.[eE].*?\.', f".E{str(self.data['process_info']['change_epi_number']).zfill(2)}.", self.data['filename']['original'])
        elif self.data['process_info']['rebuild'] == 'change_epi_numbermatch_2':
            # 날짜만 있는 원본 에피소드 삽입
            return self.data['filename']['original'].replace(f".{self.data['filename']['date']}.", f".E{str(self.data['process_info']['change_epi_number']).zfill(2)}.{self.data['filename']['date']}.")
        elif self.data['process_info']['rebuild'] == 'change_date':
            return self.data['filename']['original'].replace(f".{self.data['filename']['date']}.", f".{self.data['process_info']['change_date']}.")
            time.sleep(10)
        else:
            pass

    

    def find_meta_tmdb(self):
        from support_site import SiteTmdbFtv
        module_map = [('tmdb', SiteTmdbFtv)]
        
        for site, site_class in module_map:
            try:
                if self.data['filename']['name'] in EntityKtv.meta_cache and site in EntityKtv.meta_cache[self.data['filename']['name']]:
                    self.data['meta'] = EntityKtv.meta_cache[self.data['filename']['name']][site]
                    # 없는 것도 저장하여 중복검색 방지
                    if self.data['meta']['find']:
                        return
                site_data = site_class.search(self.data['filename']['name'])
                #logger.warning(f"{site} {d(site_data)}")
                if site_data['ret'] == 'success':
                    if len(site_data['data']) > 0 and site_data['data'][0]['score'] >= 80:
                        self.data['filename']['name'] = site_data['data'][0]['title']
                        self.find_meta()
                        if self.data['meta']['find'] == False:
                            self.data['process_info']['status'] = 'ftv'
                            self.data['process_info']['ftv_title'] = SupportFile.text_for_filename(site_data['data'][0]['title'])
                            self.data['process_info']['ftv_year'] = site_data['data'][0]['year']
                        return
            except Exception as e:
                logger.error(f"Exception:{str(e)}")
                logger.error(traceback.format_exc())


    def find_meta_season(self):
        if self.data['meta']['info']['code'][1] == 'D':
            for idx, season in enumerate(self.data['meta']['search']['series']):
                if self.data['meta']['info']['code'] == season['code']:
                    self.data['meta']['info']['season'] = idx + 1
                    return
        else:
            self.data['meta']['info']['season'] = -1




import os
import traceback

import yaml

from . import logger


class SupportYaml(object):
    @classmethod
    def write_yaml(cls, filepath, data):
        with open(filepath, 'w', encoding='utf8') as f:
            yaml.dump(data, f, default_flow_style=False, allow_unicode=True)

    @classmethod
    def read_yaml(self, filepath):
        with open(filepath, encoding='utf8') as file:
            data = yaml.load(file, Loader=yaml.FullLoader)
        return data


    @classmethod
    def copy_section(cls, source_file, target_file, section_name):
        from support import SupportFile
        try:
            if os.path.exists(source_file) == False:
                return 'not_exist_source_file'
            if os.path.exists(target_file) == False:
                return 'not_exist_target_file'
            lines = SupportFile.read_file(source_file).split('\n')
            section = {}
            current_section_name = None
            current_section_data = None

            for line in lines:
                line = line.strip()
                if line.startswith('# SECTION START : '):
                    current_section_name = line.split(':')[1].strip()
                    current_section_data = []
                if current_section_data is not None:
                    current_section_data.append(line)
                if line.startswith('# SECTION END'):
                    section[current_section_name] = current_section_data
                    current_section_name = current_section_data = None

            if section_name not in section:
                return 'not_include_section'
            
            data = '\n'.join(section[section_name])
            source_data = SupportFile.read_file(target_file)
            source_data = source_data + f"\n{data}\n"
            SupportFile.write_file(target_file, source_data)
            return 'success'
        except Exception as e: 
            logger.error(f"Exception:{str(e)}")
            logger.error(traceback.format_exc())
            return 'exception'


import traceback

from support import SupportSubprocess, logger


class SupportFfprobe:
    __ffprobe_path = 'ffprobe'

    @classmethod
    def initialize(cls, __ffprobe_path):
        cls.__ffprobe_path = __ffprobe_path
        
    @classmethod
    def ffprobe(cls, filepath, ffprobe_path=None, option=None):
        try:
            if ffprobe_path == None:
                ffprobe_path = cls.__ffprobe_path

            command = [ffprobe_path, '-v', 'quiet', '-print_format', 'json', '-show_format', '-show_streams', filepath]
            if option is not None:
                command += option
            logger.warning(' '.join(command))
            ret = SupportSubprocess.execute_command_return(command, format='json')
            return ret['log']
        except Exception as e: 
            logger.error(f"Exception:{str(e)}")
            logger.error(traceback.format_exc())    


from support import SupportSubprocess
from tool import ToolModalCommand

from .page_command import PageCommand
from .setup import *

name = 'tool'

class ModuleTool(PluginModuleBase):
    def __init__(self, P):
        super(ModuleTool, self).__init__(P, name=name, first_menu='command')
        
        self.set_page_list([PageUpload, PageCrypt, PagePython, PageCommand])



class PageUpload(PluginPageBase):
    db_default = {
        'path_upload': os.path.join(F.config['path_data'], 'upload'),
    } 

    def __init__(self, P, parent):
        super(PageUpload, self).__init__(P, parent, name='upload')



class PageCrypt(PluginPageBase):
    def __init__(self, P, parent):
        super(PageCrypt, self).__init__(P, parent, name='crypt')

        self.db_default = {
            f'{self.parent.name}_{self.name}_use_user_key': 'False',
            f'{self.parent.name}_{self.name}_user_key': '',
            f'{self.parent.name}_{self.name}_user_key': '',
        } 

class PagePython(PluginPageBase):
    def __init__(self, P, parent):
        super(PagePython, self).__init__(P, parent, name='python')
        self.db_default = {
            f'{self.parent.name}_{self.name}_name': '',
        } 

    def process_command(self, command, arg1, arg2, arg3, req):
        ret = {'ret':'success'}
        if command == 'get_freeze':
            command = ['pip', 'freeze']
            result = SupportSubprocess.execute_command_return(command)
            if result['status'] == 'finish':
                ret['data'] = []
                for tmp in result['log'].split('\n'):
                    if '==' in tmp:
                        ret['data'].append(tmp.split('=='))
            else:
                ret['ret'] = 'danger'
                ret['msg'] = "실패"
        elif command == 'upgrade':
            P.ModelSetting.set(f'{self.parent.name}_{self.name}_name', arg1)
            cmd = ['pip', 'install', '--upgrade', arg1]
            ToolModalCommand.start("pip 설치", [cmd])
        elif command == 'remove':
            cmd = ['pip', 'uninstall', '-y', arg1]
            ToolModalCommand.start("pip 삭제", [cmd])
        return jsonify(ret)



import json
import time
import traceback
from functools import wraps

from . import logger


def pt(f):
    @wraps(f)
    def wrapper(*args, **kwds):
        start = time.time()
        #logger.debug(f"FUNC START [{f.__name__}]")
        result = f(*args, **kwds)
        elapsed = time.time() - start
        logger.info(f"FUNC END [{f.__name__}] {elapsed}")
        return result
    return wrapper

default_headers = {
    'accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.9',
    'accept-language': 'ko-KR,ko;q=0.9,en-US;q=0.8,en;q=0.7',
    'user-agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/96.0.4664.110 Safari/537.36',
}


class SupportUtil(object):
    @classmethod
    def sizeof_fmt(cls, num, suffix='Bytes'):
        for unit in ['', 'K', 'M', 'G', 'T', 'P', 'E', 'Z']:
            if abs(num) < 1024.0:
                return "%3.1f %s%s" % (num, unit, suffix)
            num /= 1024.0
        return "%.1f %s%s" % (num, 'Y', suffix)

    @classmethod
    def is_arm(cls):
        try:
            ret = False
            import platform
            if platform.system() == 'Linux':
                if platform.platform().find('86') == -1 and platform.platform().find('64') == -1:
                    ret = True
                if platform.platform().find('arch') != -1:
                    ret = True
                if platform.platform().find('arm') != -1:
                    ret = True
            return ret
        except Exception as e: 
            logger.error(f"Exception:{str(e)}")
            logger.error(traceback.format_exc())
    

def dummy_func():
    pass
     
class celery(object):
    
    class task(object):
        def __init__(self, *args, **kwargs):
            if len(args) > 0:
                self.f = args[0]
    
        def __call__(self, *args, **kwargs):
            if len(args) > 0 and type(args[0]) == type(dummy_func):
                return args[0]
            self.f(*args, **kwargs)    






class SingletonClass(object):
    __instance = None
    
    @classmethod
    def __getInstance(cls):
        return cls.__instance

    @classmethod
    def instance(cls, *args, **kargs):
        cls.__instance = cls(*args, **kargs)
        cls.instance = cls.__getInstance
        return cls.__instance



class AlchemyEncoder(json.JSONEncoder):

    def default(self, obj):
        from sqlalchemy.ext.declarative import DeclarativeMeta
        if isinstance(obj.__class__, DeclarativeMeta):
            # an SQLAlchemy class
            fields = {}
            for field in [x for x in dir(obj) if not x.startswith('_') and x != 'metadata']:
                data = obj.__getattribute__(field)
                try:
                    json.dumps(data) # this will fail on non-encodable values, like other classes
                    fields[field] = data
                except TypeError:
                    fields[field] = None
            # a json-encodable dict
            return fields

        return json.JSONEncoder.default(self, obj)
